package com.ibm.sysmgt.raidmgr.cim.consumer.v2;

/* 
 * This code shamelessly stolen from the Sun WBEM SDK.
 * Refactored to support extrinsic events.
 */

import java.rmi.*;
import java.util.Enumeration;
import java.util.Vector;
import java.awt.event.*;

import javax.wbem.client.*;
import javax.wbem.client.CIMEvent;
import javax.wbem.client.CIMListener;
import javax.wbem.cim.*;

import javax.wbem.client.UserPrincipal;
import javax.wbem.client.PasswordCredential;


/**
 * Monitors for extrinsic events. Works in the default namespace root/cimv2.
 */
public class IndicationConsumer extends Thread
{
   private String host;
   private String user;
   private String password;
   private String indicationType;
   private String protocol;
   private boolean listen = true;

   public IndicationConsumer(String host, String user, String password, String indicationType, String protocol)
   {
      this.host = host;
      this.user = user;
      this.password = password;
      this.indicationType = indicationType;
      this.protocol = protocol;
   }

   public static void main(String args[])
   {
      if (args.length < 4)
      {
         System.out.println("Usage: IndicationConsumer <host> <user> <password> <indication class> [rmi|http]");
         System.out.println("   Ex: IndicationConsumer localhost joe pw ADPT_StorageRAIDAlertIndication");
         System.exit(1);
      }

      String protocol = CIMClient.CIM_RMI;
      if (args.length == 5 && args[4].equalsIgnoreCase("http"))
         protocol = CIMClient.CIM_XML;

      IndicationConsumer consumer = new IndicationConsumer(args[0], args[1], args[2], args[3], protocol);
      consumer.start();
   }

   public void run()
   {
      CIMClient cc = null;
      CIMObjectPath cop = null;
      Listener cl = null;      
      try
      {
         CIMClass cclass = null;
         CIMNameSpace cns = new CIMNameSpace(host);
         UserPrincipal up = new UserPrincipal(user);
         PasswordCredential pc = new PasswordCredential(password);         
         cc = new CIMClient(cns, up, pc, protocol);

         // Add this client as a listener for an event 
         cl = new Listener();
         cc.addCIMListener((CIMListener) cl);

         // Create a CIM_IndicationFilter instance in the target namespace
         String filterClassName = new String("CIM_IndicationFilter");
         CIMClass filterClass = cc.getClass(new CIMObjectPath(filterClassName), true, true, true, null);
         CIMInstance filterInstance = filterClass.newInstance();
         String filterString = new String("SELECT  * FROM " + indicationType);
         filterInstance.setProperty("Query", new CIMValue(filterString));
         filterInstance.setProperty("QueryLanguage", new CIMValue("WQL"));
         // NOTE: Since empty CIMObjectPath is passed in, CIMObjectPath to 
         // filterInstance is returned
         CIMObjectPath fop = cc.createInstance(new CIMObjectPath(), filterInstance);

         // Create a CIM_IndicationHandler subclass instance in the target namespace
         CIMInstance handlerInstance = cc.getIndicationHandler(null);

         // For some reason, getIndicationHandler does not set these keys for XML
         if (protocol.equalsIgnoreCase(CIMClient.CIM_XML))
         {
            handlerInstance.setProperty("Owner", new CIMValue(user));
            handlerInstance.setProperty("CreationClassName", new CIMValue(new String("CIM_IndicationHandlerCIMXML")));
            handlerInstance.setProperty("SystemName", new CIMValue(host));
         }

         // NOTE: Since empty CIMObjectPath is passed in, CIMObjectPath to 
         // handlerInstance is returned
         CIMObjectPath hop = cc.createInstance(new CIMObjectPath(), handlerInstance);

         // Create an instance of association CIM_IndicationSubscription in
         // target namespace to bind the filter to its handler. Doing so causes
         // CIMOM to invoke the target provider's activateFilter method; i.e. subscribe
         String subscriptionClassName = new String("CIM_IndicationSubscription");
         CIMClass subscriptionClass = cc.getClass(new CIMObjectPath(subscriptionClassName),
                                                  true, true, true, null);
         CIMInstance subscriptionInstance = subscriptionClass.newInstance();
         // Set association REFs to point to filter and handler instances
         subscriptionInstance.setProperty("Filter", new CIMValue(fop));
         subscriptionInstance.setProperty("Handler", new CIMValue(hop));
         // NOTE: Since empty CIMObjectPath is passed in, CIMObjectPath to 
         // subscriptionInstance is returned
         CIMObjectPath sop = cc.createInstance(new CIMObjectPath(), subscriptionInstance);

         // Wait for indication delivery         
         try
         {
            while (listen)
            {            
               System.out.println("...Waiting for event " + indicationType + ".");
               Thread.sleep(10000);
            }

         }
         catch (InterruptedException ex)
         {}                

         // Delete the subscription instance in the target namespace to 
         // unbind the filter from its handler. Doing so causes CIMOM to
         // invoke the target provider's deActivateFilter method; i.e. unsubscribe
         sop.setNameSpace("");
         cc.deleteInstance(sop);

         // Delete the filter and handler instances
         hop.setNameSpace("");
         cc.deleteInstance(hop);
         fop.setNameSpace("");
         cc.deleteInstance(fop);         
      }
      catch (CIMException ex)
      {
         System.out.println("Exception: " + ex);
         ex.printStackTrace(); 
      }

      // close session.
      try
      {
         if (cc != null)
            cc.close();
      }
      catch (CIMException ex)
      {
         ex.printStackTrace(System.out);
      }

      System.out.println("...Exiting");
   }
}

class Listener implements CIMListener
{
   public void indicationOccured(CIMEvent event)
   {
      CIMInstance indicationInstance = event.getIndication();
      System.out.println("Received indication instance: " + indicationInstance);      
   }
}

