/**
 * Title:        IBMSG ServeRAID Diagnostic Provider
 * Description:
 * Copyright:    Copyright (c) 2001,2002
 * Company:      IBM
 * Initial Date: August 30, 2001 
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/IBMSG_ServeRAID_DiagProvider.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.11
 * Extracted at            : 10:44:46 on 11/12/02
 * Newest delta applied at : 10:15:07 on 10/30/02
 * 
 * Change History:
 *              Date                            Intials/defect                          Description
 *              09102001                        RHB                                     Update provider code to the latest levels
 *              05062002                        RHB                                     Update functions to remove calls to IBMSG_ServeRAID_TestSoftware class.
 *                                                                                      Making this class static will allow us to more easily add versioning controls
 *              05242002                        RHB189412                               Add try/catch blocks around code blocks in public methods
 *
 * Notes:
 *    - Results are not supported by this provider for the getInstance, 
 *      enumInstance and get property InterFaces. The default provider will
 *      perform those functions. This provider WILL, during the runtest method,
 *      generate the result object and store it in the repository. That instance
 *      will have the key "ExecutionID" being the diag test class + the test name +
 *      the "DeviceID", making it unique. Also, in the runtest method, this provider
 *      will delete the previous results object, if it exists, to insure only one
 *      instance of this result lives in the repository. If the consumer wants a
 *      history, it must save the provious results.
 *      
 *
 */


//$$$ change package name
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;


import com.sun.wbem.cim.*;
import com.sun.wbem.client.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.provider.MethodProvider;
import com.sun.wbem.provider.PropertyProvider;
import java.io.*;
import java.util.*;
import java.lang.*;
import com.ibm.CDMProvider.*;




/** 
 * This provider supports the IBMSG ServeRAID Controller diagnostics.
 */
//$$$ change provicer name
public class IBMSG_ServeRAID_DiagProvider implements InstanceProvider,
MethodProvider
{
   /**
    * private variables 
    */
   //public static String namespace = "root/cimv2";

   private CIMOMHandle cimomhandle;

   //$$$ >>  change all class name prefixes ex: IBMSG_Widget to zxy_dev
   private static final String providername = "IBMSG_ServeRAID_DiagProvider";
   private static final String testinfoclass = "IBMSG_ServeRAID_Tests";

   private static final String dtclass = "IBMSG_ServeRAID_DiagTest";
   private static final String settingclass = "IBMSG_ServeRAID_DiagSetting";
   private static final String resultclass = "IBMSG_ServeRAID_DiagResult";
   private static final String dtforMSEclass = "IBMSG_ServeRAID_DiagTestForMSE";
   private static final String resultforMSEclass = "IBMSG_ServeRAID_DiagResultForMSE";
   private static final String resultfortestclass = "IBMSG_ServeRAID_DiagResultForTest";
   private static final String testinpkgclass = "IBMSG_ServeRAID_DiagTestInPkg";
   private static final String resultinpkgclass = "IBMSG_ServeRAID_DiagResultInPkg";
   private static final String testsoftwareassocclass = "IBMSG_ServeRAID_DiagTestSoftwareAssoc";
   private static final String settingfortestclass = "IBMSG_ServeRAID_DiagSettingForTest";


   private static final String testsoftwarename =       "IBMSG_ServeRAID_Diagnostics";
   private static final String testsoftwareelementid =  "raid_prov";
   private static final int testsoftwareelementstate = 3;
   private static final int testsoftewaretargetos = 19;
   private static final String testsoftwareversion = "1.0.0.0";

   /**
    * comment out for null if no logfile is needed
    */
   private IBMSG_ServeRAIDTest tt;
   //public static String logfile = null;
   private static final String logfile = "ServeRAID Provider.log";

   private static final String testsoftwareclass = "IBMSG_ServeRAID_TestSoftware";
   //$$$ <<


   private IBMSG_DiagUtils utils;

   //****************/
   //* constructor  */
   //****************/
   //$$$  change constructor name 
   public IBMSG_ServeRAID_DiagProvider()
   throws CIMException
   {
      try
      {

         IBMSG_DiagUtils utils = new IBMSG_DiagUtils(cimomhandle);
         utils.DEBUG_PRINT(logfile, "\n\nIn Provider constructor \n");
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }



   // methods of Interface CIMProvider (con.sun.wbem.provider)

   /**
    * The CIM Provider Interface initialize method connects the provider to 
    * the CIMOM
    * @parm cimom The CIMOM passes a handle to the provider. This
    * handle is saved for future communications with the CIMOM.
    * @throws CIMException  
    */
   public void initialize(CIMOMHandle cimom)
   throws CIMException
   {
      try
      {

         utils = new IBMSG_DiagUtils(cimomhandle);

         // logfile is used to log DEBUG_PRINT statements
         utils.DEBUG_PRINT(logfile, "\n\nIn DiagProvider.initialize\n");

         // save the cimom handle for future use
         cimomhandle = cimom;

         //$$$ change to your test class
         tt = new IBMSG_ServeRAIDTest();

         utils.DEBUG_PRINT(logfile, "Leaving DiagProvider.initialize\n");
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }


   /**
    * The CIM Provider Interface cleanup method disconnects the provider 
    * from CIMOM and terminates the provider.
    * @throws CIMException
    */
   public void cleanup()
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "In DiagProvider.cleanup\n");
   }

   // end methods of Interface CIMProvider



   // methods for Interface InstanceProvider (com.sun.wbem.provider20)


   /**
    * The Instance Provider Interface method creatInstance.
    * @return not supported by this provider.
    */
   public CIMObjectPath createInstance(CIMObjectPath cop, CIMInstance ci)
   throws CIMException
   {
      throw(new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }


   /**
    * The Instance Provider Interface method deleteInstance.
    * @return not suported by this provider.
    */
   public void deleteInstance(CIMObjectPath cp)
   throws CIMException
   {
      throw(new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }


   /**
    * This Instance Provider Interface method enumInstance returns a Vector
    * of CIMInstance(s) based on the CIMClass and object path passed in.
    * @param op CIMObjectPath to the class to be enumerated.
    * @param deep If true, deep enumeration is to be done.
    * @param cc The class reference.
    * @param localOnly If true, only the non-inherited properties
    *  are to be returned, otherwise all properties are required.
    * @return Vector of CIMInstances.
    * @throws CIMException
    */
   public Vector enumInstances(CIMObjectPath op, boolean deep,
                               CIMClass cc, boolean localOnly)
   throws CIMException
   {
      try
      {

         utils.DEBUG_PRINT(logfile, "In DiagProvider.enumInstances localonly()\n");
         return enumerateInstances(op, deep, cc, localOnly);
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }


   /**
    * This Instance Provider Interface method enumInstance returns a Vector
    * of CIMObjctPath(s) based on the CIMClass and object path passed in.
    * @param op CIMObjectPath to the class to be enumerated.
    * @param deep If true, deep enumeration is to be done.
    * @param cc The class reference.
    * @return Vector of CIMObjectPath(s)
    * @throws CIMException
    */
   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc)
   throws CIMException
   {
      try
      {

         utils.DEBUG_PRINT(logfile, "In DiagProvider.enumInstances() op: "+op+"\n");
         return enumerateNames(op, deep, cc);
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }


   /**
    * This Instance Provider Interface method execQuery handles client queries
    * for the dynamic data they provide. The provider parses the query string,
    * enumerates the instances in the class, and then filters those instances
    * to match the query string. The provider then returns a vector containing
    * the instances that match the query string. 
    *
    * @param cop The object name of the CIM Class to enumerate.
    * @param szQuery The CIM query expression.
    * @param ql The CIM query.
    * @param cc The CIM class reference.
    * @return Vector of CIMInstances matching the query filters.
    * @throws CIMException
    */
   //### ToDo - this method needs work, it is not complete
   public Vector execQuery(CIMObjectPath cop, String szQuery,int ql,CIMClass cc)
   throws CIMException
   {
      try
      {

         utils.DEBUG_PRINT(logfile, "In DiagProvider.execQuery()\n");

         // Only support WBEM Query Language level 1
         if (ql != CIMClient.WQL)
         {
            throw(new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
         }

         WQLQuery query = new WQLQuery(szQuery, cc);
         utils.DEBUG_PRINT(logfile, "   WQLQuery parsed\n");

         Vector props = query.getSelectProperties();
         utils.DEBUG_PRINT(logfile, "   returned properties\n");
         Vector results = new Vector();
         Vector v = enumerateNames(cop, true, cc);
         int i, j;

         for (i=0; i<v.size(); i++)
         {
            Vector cpv = new Vector();

            // Does this instance match the WHERE clause?
            utils.DEBUG_PRINT(logfile, "   testing WHERE clause\n");
            CIMInstance inst = (CIMInstance)v.elementAt(i);
            if (query.evaluate(inst))
            {
               utils.DEBUG_PRINT(logfile, "   matched WHERE clause\n");

               CIMInstance ci = cc.newInstance();

               // Return only the properties specified in the SELECT clause
               for (j=0; j<props.size(); j++)
               {
                  String name = ((CIMProperty)props.elementAt(j)).getName();
                  CIMProperty cp = inst.getProperty(name);
                  cpv.addElement(cp);
               }
               ci.setProperties(cpv);
               results.addElement(ci);
            }
         }
         return results;
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }


   /**
    * The Instance Provider Interface method getInstance returns the 
    * instance for the specified CIMObjectPath passed in
    * @param op the CIMObjectPath to the instance containing the
    * specified property
    * @param cc The reference class.
    * @param localOnly if true, only the non-inherited properties
    *  are to be returned, otherwise all properties are required.
    * @return The specified CIM instance.
    * @throws CIMException
    */

   // NOTE: For associations: if both REFs of the association exits you return
   //       an instance with the valid REFs. If one or both of the REFs is
   //       invalid, you have the choice of returning a REF path that is
   //       null (invalid in terms it can't be resolved to an instance) -or-
   //       you can return a null instance. I have chosen the latter

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc,
                                  boolean localOnly)
   throws CIMException
   {
      try
      {
         utils.DEBUG_PRINT(logfile, "In DiagProvider.getInstance()\n");
         return getSingleInstance(op, cc, localOnly);
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }




   /**
    * The Instance Provider Interface method setInstance.
    * @return not supported by this provider
    */
   public void setInstance(CIMObjectPath cop, CIMInstance ci)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "In DiagProvider.setInstance()\n");
      throw(new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   // * end methods for Interface InstanceProvider



   /**
    * The Method Provider Interface invokeMethod supports the methods:
    * RunTest  - This is the method use to excute the test cases.
    * ClearResults  - returns not supported by this provider.
    * DiscontinueTest  - returns not supported by this provider.
    * StartService  - returns not supported by this provider.
    * StopService  - returns not supported vy this provider.
    * 
    * @param op The path to instance of whose method gets invoked (the
    *  diag test)
    * @param methodName The method to get invoked.
    * @param inParams Input parameters to the method
    * @param outParams Output parameters from the method
    * @return CIMValue return code indicating if method execution was 
    * successful or not.
    * @throws CIMException
    */
   public CIMValue invokeMethod(CIMObjectPath op, String methodName,
                                Vector inParams, Vector outParams)
   throws CIMException
   {
      try
      {

         utils.DEBUG_PRINT(logfile, "In invokeMethod for op :"+op+"\n");

         if (op.getObjectName().equalsIgnoreCase(dtclass))
         {
            //$$$ change class name but not rt
            IBMSG_ServeRAID_RunTests rt = new IBMSG_ServeRAID_RunTests(cimomhandle, logfile);

            if (methodName.equalsIgnoreCase("RunTest"))
            {
               utils.DEBUG_PRINT(logfile, "   In invokeMethod RunTest method\n");

               return rt.runTest(op, settingclass, resultclass, dtclass,
                                 inParams, outParams);
            }


            if (methodName.equalsIgnoreCase("ClearResults"))
            {
               utils.DEBUG_PRINT(logfile, "   In invokeMethod ClearResults method\n");

               IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass, cimomhandle);

               return rt.clearResults(op, inParams, outParams, resultclass, dtclass);

            }


            if (methodName.equalsIgnoreCase("DiscontinueTest"))
            {
               utils.DEBUG_PRINT(logfile, "   In invokeMethod DiscontinueTest\n");

               //return rt.discontinueTest(op, settingclass, inParams, outParams);
               return new CIMValue(new UnsignedInt32(2));
            }


            if (methodName.equalsIgnoreCase("StartService"))
            {
               utils.DEBUG_PRINT(logfile, "   In invokeMethod StartService\n");
               // method not supported
               return new CIMValue(new UnsignedInt32(2));
            }


            if (methodName.equalsIgnoreCase("StopService"))
            {
               utils.DEBUG_PRINT(logfile, "   In invokeMethod StopService\n");
               // method not supported
               return new CIMValue(new UnsignedInt32(2));
            }
         }
         // 2 = method not supported
         return new CIMValue(new UnsignedInt32(2));
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   } 

   // end Interface MethodProvider




   /**
    * The Property Provider Interface method getPropertyValue returns the
    * CIMValue of the property specified.
    * @param op CIMObjectPath to the CIMInstance containing the property
    * @param originClass Contains the name of the class where this property 
    * was originally defined in the hierarchy.
    * @param propertyName the name of the property
    * @return the CIMValue of the specified property.
    * @throws CIMException
    */
   public CIMValue getPropertyValue(CIMObjectPath op, String originClass,
                                    String propertyName)
   throws CIMException
   {
      try
      {

         // return CIMValue of the property
         return getpropval(op, originClass, propertyName);
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }


   /**
    * The Property Provider Interface method setPropertyValue sets the value
    *  passed in to the specified property of the specified CIMObjectPath
    * @param op CIMObjectPath to the CIMInstance containing the property
    * @param originClass Contains the name of the class where this property 
    * was originally defined in the hierarchy.
    * @param propertyName the name of the property
    * @param cv the CIMValue to set the property to.
    * @return the CIMValue of the specified property.
    * @throws CIMException

    */
   public CIMValue setPropertyValue(CIMObjectPath op, String originClass,
                                    String propertyName, CIMValue cv)
   throws CIMException
   {
      try
      {

         // return CIMValue of the property
         return setpropval(op, originClass, propertyName, cv);  
      }
      catch (CIMException cimExc)
      {
         throw cimExc;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }
   }

   //  end property provider interface






   //******************************/
   //* private enumerateInstances */
   //******************************/

   /**
    * This private method is used to support enumInstance. This method
    * calls the appropriate class based on the CIMClass and CIMObjectPath
    * passed in. 
    * localOnly is ignored because there are no local only properties - they are
    * all inherited. 
    */

   // * NOTE: For associations: if both REFs of the association exists you return
   // *       an instance with the valid REFs. If one or both of the REFs is
   // *       invalid, you have the choice of returning a REF path that is
   // *       null (invalid in terms it can't be resolved to an instance) -or-
   // *       you can return a null instance. I have chosen the latter

   private Vector enumerateInstances(CIMObjectPath op, boolean deep,
                                     CIMClass cc, boolean localOnly)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "   In private enumerateInstances(), localOnly is: "+
                        localOnly+"\n");

      Vector instances = new Vector();
      Vector tempvect = new Vector();
      CIMInstance tempci = null;
      String classname;
      int i = 0, j = 0;

      // builder for diag test
      IBMSG_DiagTest dt = new IBMSG_DiagTest(cimomhandle, tt, dtclass);

      // get the classname of the object (class in this case) to be enumerated
      // The CIMClass cc that is passed in is the whole class template as in
      // the mof file, descriptions and all
      classname = op.getObjectName();



      // Diag Test class - create an instance of each test case using the 
      // data from the Tests class
      if (classname.equals(dtclass))
      {
         utils.DEBUG_PRINT(logfile, "      in enumerateInstances dtclass for index "+i+"\n");

         //  We need the dt set up but the try/catch is left as an
         //  example of how to get a stack trace in your log

         //IBMSG_DiagTest dt = null;
         //try
         //{
         //    dt = new IBMSG_DiagTest(cimomhandle, new Tests class(),
         //                                        dtclass);
         //}
         //catch (Throwable t)
         //{
         //    if (logfile != null)
         //    {
         //        try
         //        {
         //            PrintWriter out = new PrintWriter(new FileWriter(logfile, true));
         //            t.printStackTrace(out);
         //            out.close();
         //        }
         //        catch (java.io.IOException e) {};
         //    }
         //}

         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            // get the instance for the index
            instances.addElement(dt.getTestInstance(cc, i));
         }
         return instances;
      }



      // Assoc Diag Test for MSE class 
      //  - enumerate the device instances. if none exist return a null
      //    since there can be no associations with device instances.
      //  - create an associations instance of every test case for each device. 
      if (classname.equals(dtforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance DiagTestForMSE\n");
         //*  keys for assoc classes are the ant and dep ( device & test)

         // enum devices - if none return null
         Vector dev_vect = tt.enumerateDevicePaths(cimomhandle);

         if (dev_vect == null)
         {
            // there are no devices so there can be no associations
            return null;
         }

         // builder for dtmse
         IBMSG_DiagTestForMSE dtmse = new IBMSG_DiagTestForMSE(cimomhandle, tt);

         // create an association instance for each device & test combination
         for (j=0; j < dev_vect.size(); j++)
         {
            CIMObjectPath devop = (CIMObjectPath)dev_vect.elementAt(j);
            utils.DEBUG_PRINT(logfile, "      devop is: "+devop+"\n");

            CIMInstance cimInstance = cimomhandle.getInstance(devop,false);
            CIMProperty cimProperty = cimInstance.getProperty("Caption");
            CIMValue    cimValue    = cimProperty.getValue();
            String      caption     = cimValue.toString();
            utils.DEBUG_PRINT(logfile,"Caption Size"+cimValue.size());
            caption = caption.trim();
            utils.DEBUG_PRINT(logfile,"\n\n\n\n\n\n Caption is"+caption);


            try
            {
               if (caption.lastIndexOf("ServeRAID") > 0)
               {
                  if (caption.lastIndexOf("Virtual") < 0)
                  {
                     utils.DEBUG_PRINT(logfile,"Inside of if caption.lastIndexOf ServeRAID clause");
                     for (i=0; i < tt.getNumberOfTests(); i++)
                     {
                        // get the test path
                        CIMObjectPath top = dt.getTestPath(i);
                        utils.DEBUG_PRINT(logfile, "      top is: "+top+"\n");

                        instances.addElement(dtmse.dtForMSE(cc, i, devop, top));
                        utils.DEBUG_PRINT(logfile, "      intances are "+instances+"\n");
                     }
                  }
               }
            }
            catch (NullPointerException nullPointerException)
            {
            }
         }

         return instances;
      }



      //  Assoc Diag Result for MSE class
      //  - enumerate result instances, if none return null
      //  - call resformse for each combination    
      if (classname.equals(resultforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance resforMSE\n");

         IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass, cimomhandle);

         // enum results - if none return null
         Vector res_vect = dr.enumResultPaths();
         if (res_vect == null)
         {
            // there are no result instances so there can be no association 
            return null;
         }

         // enum devices - in none return null
         Vector dev_vect = tt.enumerateDevicePaths(cimomhandle);

         if (dev_vect == null)
         {
            // there are no devices so there can be no associations
            return null;
         }

         IBMSG_DiagResultForMSE rfm = new IBMSG_DiagResultForMSE(cc, cimomhandle);

         CIMInstance tmpci = cc.newInstance();

         // loop thu results            
         for (i = 0; i < res_vect.size(); i++)
         {
            CIMObjectPath res_op = (CIMObjectPath)res_vect.elementAt(i);

            // loop thu devices
            for (j = 0; j < dev_vect.size(); j++)
            {
               CIMObjectPath dev_op = (CIMObjectPath)dev_vect.elementAt(j);

               // pass the result op and the device op to resForMSE - if
               // they match, an instsance is returned else a null is returned
               tmpci = rfm.resForMSE(dev_op, res_op, dtclass);
               if (tmpci != null)
               {
                  instances.addElement(tmpci);
               }
            }
         }

         return instances;
      }




      // Assoc Result for test class
      if (classname.equals(resultfortestclass))
      {
         utils.DEBUG_PRINT(logfile, "      2 In private enumerateInstance resfortest\n");

         IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass, cimomhandle);

         // enum results - if none return null
         Vector res_vect = dr.enumResultPaths();
         if (res_vect == null)
         {
            // there are no result instances so there can be no association 
            return null;
         }

         IBMSG_DiagResultForTest rft = new IBMSG_DiagResultForTest(cc, cimomhandle);

         CIMInstance tmpci = cc.newInstance();

         // loop thu results            
         for (i = 0; i < res_vect.size(); i++)
         {
            CIMObjectPath res_op = (CIMObjectPath)res_vect.elementAt(i);

            // loop thu tests
            for (j = 0; j < tt.getNumberOfTests(); j++)
            {
               CIMObjectPath test_op = dt.getTestPath(j);

               // pass the result op and the test op to resForTest - if
               // they match, an instsance is returned else a null is returned
               tmpci = rft.resForTest(test_op, res_op);
               if (tmpci != null)
               {
                  instances.addElement(tmpci);
               }
            }
         }
         //utils.DEBUG_PRINT(logfile, "      instances are: "+instances+"\n");
         return instances;
      }



      // Assoc Test in package class
      if (classname.equals(testinpkgclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance testinpkg\n");

         // builder for test in package
         IBMSG_DiagTestInPackage tip = new IBMSG_DiagTestInPackage(cc, cimomhandle, 
                                                                   tt, logfile);

         return tip.enumerateTestInPkg(testinpkgclass, dtclass, false, dt);
      }



      // Assoc Result in package class
      if (classname.equals(resultinpkgclass))
      {
         // builder for result in package
         IBMSG_DiagResultInPackage rip = new IBMSG_DiagResultInPackage(cc,
                                                                       cimomhandle , tt, logfile);


         return rip.enumeratePackages(resultinpkgclass, resultclass, dtclass, false);
      }



      // Test Software Association class
      if (classname.equals(testsoftwareassocclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance software\n");

         IBMSG_DiagTestSoftwareAssoc tsa = new IBMSG_DiagTestSoftwareAssoc(
                                                                          cimomhandle, cc, logfile);

         //$$$ change to your class name
         //IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware(
         //                                                 cimomhandle, cc, logfile);

         // loop thru tests get assoc for each
         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            // CIMObjectPath tsop = ts.getTestSoftwarePath(testsoftwareclass, i);
            CIMObjectPath tsop = getTestSoftwarePath();
            // get the path for the index
            CIMObjectPath top = dt.getTestPath(i);

            instances.addElement(tsa.dtSoftwareAssoc(tsop, top));
         }
         return instances;
      }



      // Test Software class
      // if (classname.equalsIgnoreCase(testsoftwareclass))
      // {
      //   utils.DEBUG_PRINT(logfile, "      In private enumerateInstance testsoftware\n");

      //$$$ change to your clas name
      //  IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware(
      //                                                   cimomhandle, cc, logfile);

      //       for(i=0; i < tt.getNumberOfTests(); i++)
      //       {
      //           CIMInstance cii = ts.getTestSoftware(i);
      //          instances.addElement(cii);
      //       }

      //     return instances;           
      // }




      // Setting class
      if (classname.equals(settingclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance setting\n");

         // builder for settings
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);


         return ds.enumerateSettings(cc);
      }




      // Assoc Setting for test class
      if (classname.equals(settingfortestclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance settingfortest\n");

         IBMSG_DiagSettingForTest sft = new IBMSG_DiagSettingForTest(cimomhandle, cc,
                                                                     logfile);

         // builder for setting association
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);


         // loop thru tests get assoc for each
         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            // get setting path for index
            CIMObjectPath sop = ds.getSettingPath(i);

            // get the path for the index
            CIMObjectPath top = dt.getTestPath(i);

            instances.addElement(sft.settingForTest(sop, top));
         }
         return instances;

      }



      // return null for unknown class
      return instances = null;
   }




   //**************************/
   //* private enumerateNames */
   //**************************/
   // method to support enumerate for Names (Paths)
   private Vector enumerateNames(CIMObjectPath op, boolean deep,
                                 CIMClass cc)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "   In private enumerateNames()\n");

      Vector cops = new Vector();
      Vector tempvect = new Vector();
      CIMInstance tempop = null;
      String classname;
      int i = 0, j = 0;

      // builder for diag test
      IBMSG_DiagTest dt = new IBMSG_DiagTest(cimomhandle, tt, dtclass);

      // get the classname of the object (class in this case) to be enumerated
      // The CIMClass cc that is passed in is the whole class template as in
      // the mof file, descriptions and all
      classname = op.getObjectName();



      //  Diag Test class - create an instance of each test case using the 
      //  data from the Tests class
      if (classname.equals(dtclass))
      {
         utils.DEBUG_PRINT(logfile, "      in enumerateInstances dtclass for index "+i+"\n");


         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            // get the instance for the index
            cops.addElement(dt.getTestPath(i));
         }
         return cops;
      }



      // Assoc Diag Test for MSE class 
      //  - enumerate the device instances. if none exist return a null
      //    since there can be no associations with device instances.
      //  - create an association path of every test case for each device. 
      if (classname.equals(dtforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance DiagTestForMSE\n");
         //*  keys for assoc classes are the ant and dep ( device & test)

         // enum devices - if none return null
         Vector dev_vect = tt.enumerateDevicePaths(cimomhandle);
         //DEBUG_PRINT("         dev_vect is "+dev_vect+"\n");
         if (dev_vect == null)
         {
            // there are no devices so there can be no associations
            return null;
         }

         // builder for test for MSE
         IBMSG_DiagTestForMSE dtmse = new IBMSG_DiagTestForMSE(cimomhandle, tt);

         // create an association instance for each device & test combination
         for (j=0; j < dev_vect.size(); j++)
         {
            CIMObjectPath devop = (CIMObjectPath)dev_vect.elementAt(j);
            utils.DEBUG_PRINT(logfile, "      devop is: "+devop+"\n");

            for (i=0; i < tt.getNumberOfTests(); i++)
            {
               // get the test path
               CIMObjectPath top = dt.getTestPath(i);
               utils.DEBUG_PRINT(logfile, "      top is: "+top+"\n");

               cops.addElement(dtmse.dtForMSEPath(dtforMSEclass, devop, top));
               utils.DEBUG_PRINT(logfile, "      cops are: "+cops+"\n");
            }
         }

         return cops;
      }



      //  Assoc Diag Result for MSE class
      //  - enumerate result instances, if none return null
      //  - wnumerate device instances, if none return null
      //  - call resformse for each combination    
      if (classname.equals(resultforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance resforMSE\n");

         IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass, cimomhandle);

         // enum results - if none return null
         Vector res_vect = dr.enumResultPaths();
         if (res_vect == null)
         {
            // there are no result instances so there can be no association 
            return null;
         }

         // enum devices - in none return null
         Vector dev_vect = tt.enumerateDevicePaths(cimomhandle);
         //DEBUG_PRINT("         dev_vect is "+dev_vect+"\n");
         if (dev_vect == null)
         {
            // there are no devices so there can be no associations
            return null;
         }

         IBMSG_DiagResultForMSE rfm = new IBMSG_DiagResultForMSE(cc, cimomhandle);

         CIMObjectPath tmpop = new CIMObjectPath(resultforMSEclass);

         // loop thu results            
         for (i = 0; i < res_vect.size(); i++)
         {
            CIMObjectPath res_op = (CIMObjectPath)res_vect.elementAt(i);

            // loop thu devices
            for (j = 0; j < dev_vect.size(); j++)
            {
               CIMObjectPath dev_op = (CIMObjectPath)dev_vect.elementAt(j);

               // pass the result op and the device op to resForMSE - if
               // they match, an instsance is returned else a null is returned
               tmpop = rfm.resForMSEPath(resultforMSEclass, dev_op, res_op);
               if (tmpop != null)
               {
                  cops.addElement(tmpop);
               }
            }
         }

         return cops;
      }



      // Assoc Result for test class
      if (classname.equals(resultfortestclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance resfortest\n");

         IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass, cimomhandle);

         // enum results - if none return null
         Vector res_vect = dr.enumResultPaths();
         if (res_vect == null)
         {
            // there are no result instances so there can be no association 
            return null;
         }

         IBMSG_DiagResultForTest rft = new IBMSG_DiagResultForTest(cc, cimomhandle);

         CIMObjectPath tmpop = new CIMObjectPath(resultfortestclass);

         // loop thu results            
         for (i = 0; i < res_vect.size(); i++)
         {
            CIMObjectPath res_op = (CIMObjectPath)res_vect.elementAt(i);

            // loop thu tests
            for (j = 0; j < tt.getNumberOfTests(); j++)
            {
               CIMObjectPath test_op = dt.getTestPath(j);

               // pass the result op and the test op to resForTest - if
               // they match, an instsance is returned else a null is returned
               tmpop = rft.resForTestPath(resultfortestclass, test_op, res_op);
               if (tmpop != null)
               {
                  cops.addElement(tmpop);
               }
            }
         }
         //DEBUG_PRINT("      instances are: "+instances+"\n");
         return cops;
      }



      // Assoc Test in package class
      if (classname.equals(testinpkgclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance testinpkg\n");

         // builder for test in package
         IBMSG_DiagTestInPackage tip = new IBMSG_DiagTestInPackage(cc, cimomhandle, 
                                                                   tt, logfile);

         return tip.enumerateTestInPkg(testinpkgclass, dtclass, true, dt);
      }



      // Assoc Result in package class
      if (classname.equals(resultinpkgclass))
      {
         // builder for result in package
         IBMSG_DiagResultInPackage rip = new IBMSG_DiagResultInPackage(cc,
                                                                       cimomhandle , tt, logfile);

         return rip.enumeratePackages(resultinpkgclass, resultclass, dtclass, true);
      }





      // Test Software Association class
      if (classname.equals(testsoftwareassocclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumeratePaths software\n");

         IBMSG_DiagTestSoftwareAssoc tsa = new IBMSG_DiagTestSoftwareAssoc(
                                                                          cimomhandle, cc, logfile);

         //$$$  change the class name
         //IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware(
         //                                                cimomhandle, cc, logfile);

         utils.DEBUG_PRINT(logfile, "   In dt soft assoc\n");


         // loop thru tests get assoc for each
         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            utils.DEBUG_PRINT(logfile, "   i is: "+i+"\n");

            //CIMObjectPath tsop = ts.getTestSoftwarePath(testsoftwareclass, i);
            CIMObjectPath tsop = getTestSoftwarePath();
            utils.DEBUG_PRINT(logfile, "   tsop is: "+tsop+"\n");


            // get the path for the index
            CIMObjectPath top = dt.getTestPath(i);

            utils.DEBUG_PRINT(logfile, "   top is: "+top+"\n");

            cops.addElement(tsa.dtSoftwareAssocPath(testsoftwareassocclass,
                                                    tsop, top));
         }
         return cops;
      }





      // Test Software class
      // if (classname.equalsIgnoreCase(testsoftwareclass))
      // {
      //$$$ change the class name 
      //    IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware(
      //                                                      cimomhandle, cc, logfile);

      //     for(i = 0; i < tt.getNumberOfTests(); i++)
      //     {
      //        CIMObjectPath cop = ts.getTestSoftwarePath(testsoftwareclass, i);
      //         cops.addElement(cop);
      //    }

      //    return cops;   
      //}





      // Setting class
      if (classname.equals(settingclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateNames for setting\n");

         // builder for diag settings
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);

         return ds.enumerateSettingPaths();
      }





      // Assoc Setting for test class
      if (classname.equals(settingfortestclass))
      {
         // utils.DEBUG_PRINT(logfile, "      In private enumerateInstance settingfortest\n");

         IBMSG_DiagSettingForTest sft = new IBMSG_DiagSettingForTest(cimomhandle, cc,
                                                                     logfile);

         // builder for settings
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);


         // loop thru tests get assoc for each
         for (i=0; i < tt.getNumberOfTests(); i++)
         {
            // get setting path for index
            CIMObjectPath sop = ds.getSettingPath(i);

            // get the path for the index
            CIMObjectPath top = dt.getTestPath(i);

            cops.addElement(sft.settingForTestPath(settingfortestclass, sop, top));
         }
         return cops;

      }



      // return null for unknown class
      return cops = null;
   }







   //*********************
   // support getInstance
   //*********************
   private CIMInstance getSingleInstance(CIMObjectPath op, CIMClass cc,
                                         boolean localOnly)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "   in private getSingleInstance/n");

      CIMInstance ci = cc.newInstance();

      int i = 0, j = 0;

      // builder for diag test
      IBMSG_DiagTest dt = new IBMSG_DiagTest(cimomhandle, tt, dtclass);


      // get the classname of the object (class in this case) to be enumerated
      // The CIMClass cc that is passed in is the whole class template as in
      // the mof file, descriptions and all
      String classname = op.getObjectName();



      //  Diag Test class - create an instance of each test case using the 
      //  data from the Tests class
      if (classname.equals(dtclass))
      {
         utils.DEBUG_PRINT(logfile, "      in enumerateInstances dtclass for index "+i+"\n");

         // get the instance for the index
         i = tt.getTestIndex(op, "Name");
         return ci =dt.getTestInstance(cc, i);
      }



      // Assoc Diag Test for MSE class 
      //  - enumerate the device instances. if none exist return a null
      //    since there can be no associations with device instances.
      //  - create an associations instance of every test case for each device. 
      if (classname.equals(dtforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance DiagTestForMSE\n");
         //*  keys for assoc classes are the ant and dep ( device & test)

         // enum devices - if none return null
         CIMObjectPath devop = utils.opFromRef(op, "Dependent");
         CIMInstance devci = cimomhandle.getInstance(devop, false);

         if (devci == null)
         {
            // there is no device so there can be no association
            return null;
         }

         CIMObjectPath testop = utils.opFromRef(op, "Antecedent");
         int testi = tt.getTestIndex(testop, "Name");

         if (testi == 0xFF)
         {
            // there is no device so there can be no association
            return null;
         }

         // builder for test for MSE
         IBMSG_DiagTestForMSE dtmse = new IBMSG_DiagTestForMSE(cimomhandle, tt);

         // since the device does exist and the test name is ok, just return
         // the op that was passed in
         return ci = dtmse.dtForMSE(cc, testi, devop, testop);
      }





      // Assoc Diag Result for MSE class
      //  - enumerate result instances, if none return null
      //  - wnumerate device instances, if none return null
      //  - call resformse for each combination    
      if (classname.equals(resultforMSEclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance resforMSE\n");

         // enum results - if none return null
         CIMObjectPath resop = utils.opFromRef(op, "Result");
         CIMInstance resci = cimomhandle.getInstance(resop, false);

         if (resci == null)
         {
            // there is no result instance so there can be no association 
            return null;
         }

         // enum devices - in none return null
         CIMObjectPath devop = utils.opFromRef(op, "bSystemElement");
         CIMInstance devci = cimomhandle.getInstance(devop, false);

         if (devci == null)
         {
            // there is no device so there can be no association
            return null;
         }

         IBMSG_DiagResultForMSE rfm = new IBMSG_DiagResultForMSE(cc, cimomhandle);

         return ci = rfm.resForMSE(devop, resop, dtclass);

      }





      // Assoc Result for test class
      if (classname.equals(resultfortestclass))
      {
         utils.DEBUG_PRINT(logfile, "      2 In private enumerateInstance resfortest\n");

         // enum results - if none return null
         CIMObjectPath resop = utils.opFromRef(op, "DiagnosticResult");
         CIMInstance resci = cimomhandle.getInstance(resop, false);

         if (resci == null)
         {
            // there is no result instance so there can be no association 
            return null;
         }

         CIMObjectPath testop = utils.opFromRef(op, "DiagnosticTest");
         int testi = tt.getTestIndex(testop, "Name");

         if (testi == 0xFF)
         {
            // there is no device so there can be no association
            return null;
         }

         IBMSG_DiagResultForTest rft = new IBMSG_DiagResultForTest(cc, cimomhandle);
         // since the device does exist and the test name is ok, just return
         // the op that was passed in
         return ci = rft.resForTest(testop, resop);

      }



      // Assoc Test in package class
      if (classname.equals(testinpkgclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance testinpkg\n");

         // builder for test in package
         IBMSG_DiagTestInPackage tip = new IBMSG_DiagTestInPackage(cc, cimomhandle, 
                                                                   tt, logfile);


         CIMObjectPath opg = utils.opFromRef(op, "Groupcomponent");
         CIMObjectPath opp = utils.opFromRef(op, "PartComponent");

         return ci = tip.testInPkg(opg, opp);
      }



      // Assoc Result in package class
      if (classname.equals(resultinpkgclass))
      {
         // builder for result in package
         IBMSG_DiagResultInPackage rip = new IBMSG_DiagResultInPackage(cc,
                                                                       cimomhandle , tt, logfile);

         CIMObjectPath opp = utils.opFromRef(op, "PackageResult");
         CIMObjectPath opr = utils.opFromRef(op, "Result");

         return ci = rip.resInPkg(opp, opr);
      }



      // Assoc Test Software class
      if (classname.equals(testsoftwareassocclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance software\n");

         IBMSG_DiagTestSoftwareAssoc tsa = new IBMSG_DiagTestSoftwareAssoc(
                                                                          cimomhandle, cc, logfile);

         CIMObjectPath opant = utils.opFromRef(op, "Antecedent");
         CIMObjectPath opdep = utils.opFromRef(op, "Dependent");

         return ci = tsa.dtSoftwareAssoc(opant, opdep);
      }




      // Test Software class
      //if (classname.equalsIgnoreCase(testsoftwareclass))
      //{
      //$$$ change the class name
      // IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware(
      //                                                 cimomhandle, cc, logfile);

      //String testname = utils.keyValFromOp(op, "SoftwareElementID");
      // i = tt.getTestIndex(testname);

      // return ci = ts.getTestSoftware(i);
      //}





      // Setting class
      if (classname.equals(settingclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance setting\n");

         // builder for settings
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);

         return ds.getSettings(op, cc);
      }




      // Assoc Setting for test class
      if (classname.equals(settingfortestclass))
      {
         utils.DEBUG_PRINT(logfile, "      In private enumerateInstance settingfortest\n");

         IBMSG_DiagSettingForTest sft = new IBMSG_DiagSettingForTest(cimomhandle, cc,
                                                                     logfile);

         // builder for settings
         IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingclass, cimomhandle,
                                                        tt);

         CIMObjectPath opel = utils.opFromRef(op,"Element");
         i = tt.getTestIndex(opel, "Name");

         // get setting path for index
         CIMObjectPath sop = ds.getSettingPath(i);

         // get the path for the index
         CIMObjectPath top = dt.getTestPath(i);

         return ci = sft.settingForTest(sop, top);

      }



      // return null for unknown class
      return ci = null;

   }





   //*****************************************************************/
   //* methods to support Interface getproperty for PropertyProvider */
   //*****************************************************************/
   private CIMValue getpropval(CIMObjectPath op, String originClass,
                               String propertyName)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "in getpropval for prop "+propertyName+"\n");

      CIMValue value = null;
      String classname = op.getObjectName();   



      // Diag Test class
      if (classname.equals(dtclass))
      {
         utils.DEBUG_PRINT(logfile, "   in class dtclass\n");

         // get the index of the test case
         int index = tt.getTestIndex(op, "Name");

         if ("SystemCreationClassName".equalsIgnoreCase(propertyName))
            return utils.getSystemCreationClassName(dtclass);

         if ("SystemName".equalsIgnoreCase(propertyName))
            return utils.getSystemName(dtclass);

         if ("CreationClassName".equalsIgnoreCase(propertyName))
            return utils.getCreationClassName(dtclass);

         if ("Name".equalsIgnoreCase(propertyName))
            return(new CIMValue(tt.getTestName(index)));

         if ("Description".equalsIgnoreCase(propertyName))
            return tt.getDescription();

         if ("ResourcesUsed".equalsIgnoreCase(propertyName))
            return tt.getResourcesUsed(index);

         if ("StartMode".equalsIgnoreCase(propertyName))
            return tt.getStartMode();

         if ("Characteristics".equalsIgnoreCase(propertyName))
            return tt.getCharacteristics(index);

         if ("Caption".equalsIgnoreCase(propertyName))
            return tt.getCaption(index);

         if ("InstallDate".equalsIgnoreCase(propertyName))
            return null;

         if ("OtherCharacteristics".equalsIgnoreCase(propertyName))
            return tt.getOtherCharacteristicDescription(index);

         if ("IsInUse".equalsIgnoreCase(propertyName))
            return tt.getIsInUse(index);

         if ("Started".equalsIgnoreCase(propertyName))
            return tt.getStarted(index);

         if ("Status".equalsIgnoreCase(propertyName))
            return tt.getStatus();

         throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);

      }     // end if dtclass


      // Diag Test for MSE class
      if (classname.equals(dtforMSEclass))
      {
         CIMObjectPath testop = utils.opFromRef(op, "Antecedent");

         int testindex = tt.getTestIndex(testop, "Name");
         //DEBUG_PRINT("         dev_vect is "+dev_vect+"\n");
         if (testindex == 0xFF)
         {
            // there is no device so there can be no association
            return null;
         }

         // isExclusiveForMSE property
         if ("isExclusiveForMSE".equalsIgnoreCase(propertyName))
            return tt.getIsExclusiveForMSE(testindex);

         // EstimatedTime OfPerforming property
         if ("EstimatedTimeOfPerforming".equalsIgnoreCase(propertyName))
            return tt.getEstimatedTimeOfPerforming(testindex);

         throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);

      }



      // Diag Test for MSE class
      //if (classname.equals(testsoftwareclass))
      //{
      //$$$
      //IBMSG_ServeRAID_TestSoftware ts = new IBMSG_ServeRAID_TestSoftware();

      //int index = tt.getTestIndex(op, "Name");


      //if ("Name".equalsIgnoreCase(propertyName))
      //{
      //    return ts.getName();
      //}


      //if ("SoftwareElementID".equalsIgnoreCase(propertyName))
      //{
      //    return ts.getSoftwareElementID(index);
      //}


      //if ("SoftwareElementState".equalsIgnoreCase(propertyName))
      //{
      // return ts.getSoftwareElementState();
      //}


      //if ("TargetOperatingSystem".equalsIgnoreCase(propertyName))
      //{
      //return ts.getTargetOperatingSystem();
      //}


      // if ("Version".equalsIgnoreCase(propertyName))
      //{
      //return ts.getVersion(index);
      //}


      //if ("Manufacturer".equalsIgnoreCase(propertyName))
      //{
      // return ts.getManufacturer();
      //}


      //if ("Caption".equalsIgnoreCase(propertyName))
      //{
      //return ts.getCaption();
      //}


      // if ("Description".equalsIgnoreCase(propertyName))
      //{
      //return ts.getDescription();
      //}


      // if ("InstallDate".equalsIgnoreCase(propertyName))
      //{
      //return ts.getTSInstallDate(index);
      //}


      //throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);
      //}



      // return null for unknown class
      throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);

   }






   //*****************************************************************/
   //* methods to support Interface setproperty for PropertyProvider */
   //*****************************************************************/
   private CIMValue setpropval(CIMObjectPath op, String originClass,
                               String propertyName, CIMValue cv)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "in setpropval for prop "+propertyName+"\n");

      String classname = op.getObjectName();   


      // Setting Diag Test class
      if (classname.equals(settingclass))
      {
         // get the test index from the Setting op SettingID property
         String setID = utils.keyValFromOp(op, "SettingID");

         // parse SettingID for the test name
         StringTokenizer st = new StringTokenizer(setID, "|", false);

         // first token is the class name
         String tokenstr = st.nextToken();
         // second token is the test name
         String testname = st.nextToken();

         // get the test index
         int testindex = tt.getTestIndex(testname);

         if ("QuickMode".equalsIgnoreCase(propertyName))
            return tt.setQuickMode(testindex, cv);


         if ("HaltOnError".equalsIgnoreCase(propertyName))
            return tt.setHaltOnError(testindex, cv);


         if ("ReportSoftErrors".equalsIgnoreCase(propertyName))
            return tt.setReportSoftErrors(testindex, cv);


         if ("ReportStatusMessages".equalsIgnoreCase(propertyName))
            return tt.setReportStatusMessages(testindex, cv);


         if ("TestWarningLevel".equalsIgnoreCase(propertyName))
            return tt.setTestWarningLevel(testindex, cv);


         if ("PercentOfTestCoverage".equalsIgnoreCase(propertyName))
            return tt.setPercentOfTestCoverage(testindex, cv);

      }

      throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);
   }


   /**
    * This method builds an object path to a test software instance.
    * @param testsoftwareclass This is the name of the test software class for 
    * this parovider
    * @paramindex The index to the test case.
    * @return CIMObjectPath The path for the test software object.
    */
   public CIMObjectPath getTestSoftwarePath()
   throws CIMException
   {
      try
      {

         CIMObjectPath tsop = new CIMObjectPath(testsoftwareclass);
         tsop.addKey("Name", new CIMValue(testsoftwarename));
         tsop.addKey("SoftwareElementID", new CIMValue(testsoftwareelementid));
         tsop.addKey("SoftwareElementState", new CIMValue(new UnsignedInt16(testsoftwareelementstate)));
         tsop.addKey("TargetOperatingSystem", new CIMValue(new UnsignedInt16(testsoftewaretargetos)));
         tsop.addKey("Version", new CIMValue(testsoftwareversion));

         return tsop;
      }
      catch (Exception exc)
      {
         CIMException cimExc = new CIMException(CIMException.CIM_ERR_FAILED);
         throw cimExc;
      }

   }

   //********/
   //* MAIN */
   //********/
   // Main method
   public static void main(String[] args)
   {
      // In Linux the provider is instantiated by cimom and is persistent.
      // Main and Finalize will never be called although they may in the future.
      // In Windows the provider is instantiated by cimom and is unloaded if it
      // is not called for 5 minutes (in reality it is more like 2-3 minutes)
      // When called again, the constructor and initialize are invoked.
   }
}

