/**
 * FileName:        IBMSG_ServeRAID_RunTests.java
 * Description:	 
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 * Initial Date:  August 30, 2001 
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/IBMSG_ServeRAID_RunTests.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.15
 * Extracted at            : 10:44:46 on 11/12/02
 * Newest delta applied at : 10:35:11 on 4/18/02
 * Notes:
 *  
 * Change History:
 *   Date:			Intials:		Change:
 *   09102001			RHB			Update provider code to the latest levels
 *   09112001			RHB			Update executeMethod routine to reflect RunLightPath test structure
 */

//$$$ change package name to same one as the provider
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;


import com.sun.wbem.cim.*;
import com.sun.wbem.client.*;
import java.io.*;
import java.util.*;
import java.lang.*;
import com.ibm.CDMProvider.*;



/**
 * This class does the actual test invocation, builds the result object and
 * stores it in the repository.
 */
//$$$ change the class name
public class IBMSG_ServeRAID_RunTests
{
   private CIMOMHandle cimomhandle;
   private String logfile;

   private String dut;
   private int testidx;
   private String testname;
   private CIMObjectPath rop;
   private Results r;
   private String str;
   private boolean packageFlag = false;

   // private stuff for settings
   private Vector sv = new Vector();
   private short sTestWarningLevel;
   private boolean sReportSoftErrors;
   private boolean sReportStatusMessages;
   private boolean sHaltOnError;
   private boolean sQuickMode;
   private byte sPercentOfTestCoverage;

   private IBMSG_DiagUtils utils;




   /**
    * The constructor saves the parametrs passed in by the provider.
    * @param cimomhandle This is the handle to the CIMOM passed in from the 
    * calling provider.
    * @param logfile This is the name of the debug log file.
    * @throws CIMException
    */
   //$$$ change the conatructor name 
   public IBMSG_ServeRAID_RunTests(CIMOMHandle cimomhandle, String logfile)
   throws CIMException
   {
      this.cimomhandle = cimomhandle;
      this.logfile = logfile;
      utils = new IBMSG_DiagUtils(cimomhandle);
   }




   /**
    * <P>This method extracts the InParams, checks to see if this test is exclusive and
    * if this is a package test. If it is a package test this method builds the 
    * package result object, invokes the tests, aggragates those test results into 
    * the result object and store it in the repository. If this is an individual 
    * test, this method builds a result object, runs the test, fills in the result
    *  and store it in the reopsitory.</P>
    * @param op <li>This is the referenced object path.</li>
    * @param settingclass <li>This is the seeting class name for this provider.</li>
    * @param resultclass <li>This is the result class name for this provider.</li>
    * @param dtclass <li>This is the name of the diagnostic test class for this provider.</li>
    * @param inParam <P>This is a vector with the first element being the object path to 
    * the MSE (device) to test and the second element being an object path to the
    * test settings which are optional. Even though settings are optional there MUST be
    * something in the inParam or CIMOM rejects it.</P>
    * @param outParam <li>This method stores the result objects path in this vector which
    * gets returned to the caller.</li>
    * @return CIMValue of whether the method (not the test) was successful (0), anything
    * else is a failure. Check the result object for the test status.
    * @throws CIMException
    */

   //
   // RESTRICTION: For this method, running package tests limits the tests
   //              to being run serially ( one must finish before the next
   //              test can run). A redesign is required to allow all the 
   //              tests to be kicked off at the same time.
   //              Also, nesting of package test is not allowed, ie. a
   //              package test cannot contain another package test.
   //
   // ###NOTE: Since inparams 2 is looking for a REF to a setting object,
   //          there must be something in that inparam or CIMOM kicks it out
   //          as a bad param. It can be the word "null", or garbage. This
   //          provider looks for the setting object it expects for the test
   //          it is performing, anything else and it uses default params.

   public CIMValue runTest(CIMObjectPath op, String settingsclass, String resultclass,
                           String dtclass, Vector inParams, Vector outParams)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "   In runTest method inParams are: "+inParams+"\n");

      //$$$ change only the class name not tt
      IBMSG_ServeRAIDTest tt = new IBMSG_ServeRAIDTest();

      // CHECK inParams

      int vnum = inParams.size();

      if (inParams.isEmpty() || vnum < 2)
         throw(new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER));

      utils.DEBUG_PRINT(logfile, "      inParams 1 is: "+inParams.elementAt(0)+"\n");

      // inParams(0) is a CIMValue of the device reference
      CIMValue cv = (CIMValue)inParams.elementAt(0);

      // the value of CIMValue inParams(0) is the CIMObjectPath of the device
      CIMObjectPath dutop = (CIMObjectPath)cv.getValue();

      // the device under test string value
      dut = utils.keyValFromOp(dutop, "DeviceID");

      /**
       * diagnostic test property: Status
       * Since the SCSI Tape test dll figures out the status of the dut, the 
       * provider does not have to. If you prefere to have the provider handle 
       * this function, here might be a good place to do it.
       * Get the device status information from the device instance reference 
       * by the CIMObjectPath above. could create a method in your xxtests class
       * This is  status for the device NOT the test
       * valid values are: string = {"OK", "Error", "Degraded", "Unknown",
       * "Pred Fail", "Starting", "Stopping", "Service", "Stressed", "NonRecover",
       * "No Contact", "Lost Comm"} (inherited from CIM_ManagedSystemElement)
       */

      // get the test index 
      testidx = tt.getTestIndex(op, "Name");

      if (testidx == 0xFF)
      {
         // 0xFF means the test case name was not found
         throw(new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER));

      }


      // is this test exclusive
      Boolean b = tt.getIsExclusive(testidx);
      if (b == Boolean.TRUE)
      {
         boolean bt = tt.getExclusiveTestRunning(testidx);
         if (bt == true)
         {
            return new CIMValue(new UnsignedInt32(1000));
         }
         else
         {
            tt.setExclusiveTestRunning(testidx, true);
         }
      }

      // is this test exclusive for mse
      CIMValue cvv = tt.getIsExclusiveForMSE(testidx);
      if ((Boolean)cvv.getValue() == Boolean.TRUE)
      {
         boolean bbt = tt.getExclusiveTestToMSERunning(testidx, dut);
         if (bbt == true)
         {
            return new CIMValue(new UnsignedInt32(2000));
         }
         else
         {
            int ret = tt.setExclusiveTestToMSERunning(testidx, dut, true);
            if (ret != 0)
               return new CIMValue(new UnsignedInt32(4000));
         }
      }


      // get test name
      testname = tt.getTestName(testidx);
      String pkgtestname = testname;

      /**
       * Is there a setting object passed in? This method will check for
       * a setting object, The consumer should have created a setting
       * object and passed in the reference. Anything else (even a
       * misspelled setting object) will use the default settings.
       */
      //utils.DEBUG_PRINT(logfile, "      inParams 2 is: "+inParams.elementAt(1)+"\n");
      CIMValue setcv = (CIMValue)inParams.elementAt(1);

      // the value of the CIMValue inParams(1) is the CIMObjectPath to settings
      CIMObjectPath setop = (CIMObjectPath)setcv.getValue();
      utils.DEBUG_PRINT(logfile, "         setop is: "+setop+"\n");



      IBMSG_DiagSettings ds = new IBMSG_DiagSettings(settingsclass, cimomhandle,tt);

      // get a list of settings this test supports
      sv = tt.getSettingSupportNames(testidx);

      // set the local appliable settings
      for (int j = 0; j < sv.size(); j++)
      {
         if ("QuickMode".equalsIgnoreCase((String)sv.elementAt(j)))
            sQuickMode = tt.getQuickMode(testidx);

         if ("HaltOnError".equalsIgnoreCase((String)sv.elementAt(j)))
            sHaltOnError = tt.getHaltOnError(testidx);

         if ("ReportSoftErrors".equalsIgnoreCase((String)sv.elementAt(j)))
            sReportSoftErrors = tt.getReportSoftErrors(testidx);

         if ("ReportStatusMessages".equalsIgnoreCase((String)sv.elementAt(j)))
            sReportStatusMessages = tt.getReportStatusMessages(testidx);

         if ("TestWarningLevel".equalsIgnoreCase((String)sv.elementAt(j)))
            sTestWarningLevel = tt.getTestWarningLevel(testidx);

         if ("PercentOfTestCoverage".equalsIgnoreCase((String)sv.elementAt(j)))
            sPercentOfTestCoverage = tt.getPercentOfTestCoverage(testidx);

      }

      utils.DEBUG_PRINT(logfile, "      sv is: "+sv+"\n");
      // END CHECKING inParams



      // is this a package test?
      if (tt.getIsPackage(testidx) == Boolean.TRUE)
      {
         utils.DEBUG_PRINT(logfile, "      in IsPackage code \n");

         packageFlag = true;

         IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass,cimomhandle);

         CIMObjectPath roppkg = dr.genResultPath (dtclass, testname, dut);
         utils.DEBUG_PRINT(logfile, "      roppkg is: "+roppkg+"\n");

         // delete any previous instance of this result
         dr.deleteDiagResult(roppkg);

         // get an instance of the diag result class to put the test results in
         CIMInstance ci = dr.genResultInstance(testidx, dut, dtclass, testname);

         // copy the setting used by this test into the result object
         setSettingsInResult(ci);

         // indicate test in on progress
         ci.setProperty("TestState", new CIMValue(new UnsignedInt16(4)));

         // set the Test start time property
         ci.setProperty("TestStartTime", new CIMValue(new CIMDateTime()));

         // set the Test start time property
         ci.setProperty("IsPackage", new CIMValue(Boolean.TRUE));

         tt.setIsInUse(testidx, true);


         Vector pts = new Vector();
         pts = tt.getTestPackage(testidx);

         boolean anyFailure = false;
         Vector pkgv = new Vector();

         for (int j = 1; j < pts.size(); j++)
         {
            utils.DEBUG_PRINT(logfile,"      in package test loop\n");

            testidx = ((UnsignedInt16)pts.elementAt(j)).intValue();
            testname = tt.getTestName(testidx);

            utils.DEBUG_PRINT(logfile,"      test is: "+testname+"\n");

            executeTest(resultclass, dtclass, settingsclass);

            if (r.statusCode != 1)
            {
               // test failed
               anyFailure = true;
            }

            pkgv.addElement(str);
         }


         String str1 = new CIMDateTime().toString()+" | "+pkgtestname+" | ";

         /**
          * TestState Values{"Unknown"(0), "Other"(1), "Passed"(2),
          *                  "Failed"(3), "In Progress"(4), "Stopped"(5)}
          * For SCSI Tape Presence test statuscode 1 = passed,
          *     anything else is failed.
          */
         if (anyFailure == false)
         {
            ci.setProperty("TestState", new CIMValue(new UnsignedInt16(2)));
            str1 = str1 + "Package test PASSED";
         }
         else
         {
            ci.setProperty("TestState", new CIMValue(new UnsignedInt16(3)));
            str1 = str1 + "Package Test FAILED";
         }

         pkgv.addElement(str1);

         ci.setProperty("TestResults", new CIMValue(pkgv,
                                                    CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY)));

         // set the percent complete property to 100 (finished)
         short percent = 100;
         ci.setProperty("PercentComplete", new CIMValue(new UnsignedInt8(percent)));

         // TestCompletionTime
         CIMDateTime dtime = new CIMDateTime();

         // put the test completion time into the result instance
         ci.setProperty("TestCompletionTime", new CIMValue(dtime));

         // indicate last time this instance was updated
         ci.setProperty("TimeStamp", new CIMValue(dtime));

         // put the instance in the CIM repository
         cimomhandle.createInstance(roppkg, ci);

         tt.setIsInUse(testidx, false);

         // Since we now know the result op, stuff it is the outparams.
         // To do so, we need to make a REF type out of it
         CIMDataType dt1 = new CIMDataType(resultclass);
         CIMValue val1 = new CIMValue(roppkg, dt1);
         outParams.addElement(val1);

         return new CIMValue(new UnsignedInt32(0));
      }


      executeTest(resultclass, dtclass, settingsclass); 

      // Since we now know the result op, stuff it in the outParams.
      // To do so, we need to make a REF type out of it
      CIMDataType dt = new CIMDataType(resultclass);
      CIMValue val = new CIMValue(rop, dt);
      outParams.addElement(val);
      //utils.DEBUG_PRINT(logfile, "         outParams are: "+outParams+"\n");

      return new CIMValue(new UnsignedInt32(0));
   }







   /**
    * <P> This class will instantiate a RunServeRAIDTest object and execute the requested 
    *     test via the created object.  The result object is populated after the test
    *     has returned.</P>
    *@param resultclass
    *@param dtclass
    *@param settingsclass
    */
   private CIMValue executeTest(String resultclass, String dtclass, String settingsclass)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "   In executeTest method \n");

      /**
       * Since the results of the test are kept in a static instance
       * (in the repository) we need to get the result object path,
       * then delete the previous isntance of this tests results
       */

      IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass,cimomhandle);


      rop = dr.genResultPath (dtclass, testname, dut);
      utils.DEBUG_PRINT(logfile, "      rop is: "+rop+"\n");

      // delete any previous instance of this result if not package
//        if (packageFlag == false)
      dr.deleteDiagResult(rop);

      // get an instance of the diag result class to put the test results in
      CIMInstance ci = dr.genResultInstance(testidx, dut, dtclass, testname);

      // copy the settings used by this test into the result object
      setSettingsInResult(ci);


      // indicate test in on progress
      ci.setProperty("TestState", new CIMValue(new UnsignedInt16(4)));

      // set the Test start time property
      ci.setProperty("TestStartTime", new CIMValue(new CIMDateTime()));

      //$$$ change only the class name not tt
      IBMSG_ServeRAIDTest tt = new IBMSG_ServeRAIDTest();

      tt.setIsInUse(testidx, true);

      utils.DEBUG_PRINT(logfile, "   In runTest, method calling test\n");
      utils.DEBUG_PRINT(logfile, "   dut is: "+dut+" testname is: "+testname+"\n");

      r = new Results();

      try
      {
         RunServeRAIDTest runServeRAIDTest = new RunServeRAIDTest();
         runServeRAIDTest.runTest(testidx,dut,r);

      }
      catch (Exception anyException)
      {
         r.statusCode = r.RESULT_STOPPED;

         utils.DEBUG_PRINT(logfile,"THrew an exception...");
         utils.DEBUG_PRINT(logfile,anyException.toString());
      }



      // return from test   
      utils.DEBUG_PRINT(logfile, "            return from test\n");

      // TestCompletionTime
      CIMDateTime dtime = new CIMDateTime();

      // put the test completion time into the result instance
      ci.setProperty("TestCompletionTime", new CIMValue(dtime));

      /**
       * Create a vector to hold the test result property.
       */
      Vector v;
      if (!r.details.isEmpty())
      {
         v  = r.details;
      }
      else
      {
         utils.DEBUG_PRINT(logfile,"No details...");
         v = new Vector();
      }

      // set the percent complete property to 100 (finished)
      short percent = 100;
      ci.setProperty("PercentComplete", new CIMValue(new UnsignedInt8(percent)));

      // if no results return with failed and msg
      if (r == null)
      {
         utils.DEBUG_PRINT(logfile, "            return invalid test\n");
         str = str+"No results returned from test";
         v.addElement(str);
         v.addElement(r.details);
         ci.setProperty("TestResults", new CIMValue(v,
                                                    CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY)));
         ci.setProperty("TestState", new CIMValue(new UnsignedInt16(3)));

         cimomhandle.createInstance(rop, ci);

         return new CIMValue(new UnsignedInt32(0));
      }


      // put the results into the test results property
      if (r.firmwareLevel != null)
      {
         ci.setProperty("FirmwareLevel",   new CIMValue(r.firmwareLevel,
                                                        CIMDataType.getPredefinedType(CIMDataType.STRING)));
         utils.DEBUG_PRINT(logfile,"FirmwareLevel:\t" + r.firmwareLevel);
      }

      if (r.statusCode != r.RESULT_PASSED)
      {
         if (r.errorCode != null)
         {
            ci.setProperty("ErrorCode",       new CIMValue(r.errorCode,
                                                           CIMDataType.getPredefinedType(CIMDataType.STRING)));
            utils.DEBUG_PRINT(logfile,"ERROR_CODE:\t"+r.errorCode);
         }
      }

      if (r.deviceLocation != null)
      {
         ci.setProperty("DeviceLocation",  new CIMValue(r.deviceLocation,
                                                        CIMDataType.getPredefinedType(CIMDataType.STRING)));
         utils.DEBUG_PRINT(logfile,"DEVICELOCATION:\t"+r.deviceLocation);
      }

      if (!v.isEmpty())
      {
         ci.setProperty("TestResults",     new CIMValue(v,
                                                        CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY)));
      }
      else
      {
         utils.DEBUG_PRINT(logfile,"No test results returned...");
      }

      if (!r.actionOnFailure.isEmpty())
      {
         ci.setProperty("ActionOnFailure", new CIMValue(r.actionOnFailure,
                                                        CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY)));
      }
      else
      {
         utils.DEBUG_PRINT(logfile,"No actions to take...");
      }


      ci.setProperty("TestState", new CIMValue(new UnsignedInt16(r.statusCode)));

      // indicate last time this instance was updated
      ci.setProperty("TimeStamp", new CIMValue(dtime));
      utils.DEBUG_PRINT(logfile,"  Printed the TimeStamp");

      // put the instance in the CIM repository if this test is not part of
      // a package

//        if (packageFlag == false)
//        {
      utils.DEBUG_PRINT(logfile," Creating result object path...");
      cimomhandle.createInstance(rop, ci);
      utils.DEBUG_PRINT(logfile, "         instance set for rop: "+rop+"\n");
//        }

      tt.setIsInUse(testidx, false);

      // reset is this test exclusive
      Boolean b1 = tt.getIsExclusive(testidx);
      if (b1 == Boolean.TRUE)
         tt.setExclusiveTestRunning(testidx, false);

      // reset is this test exclusive for mse
      CIMValue cvv1 = tt.getIsExclusiveForMSE(testidx);
      if ((Boolean)cvv1.getValue() == Boolean.TRUE)
         tt.setExclusiveTestToMSERunning(testidx, dut, false);


      // return a CIMValue for the method (NOT the test case)
      // values are: 0 = OK, 1 = Unspecified Error,
      //             2 = Not Implemented, 3 = Out Of Resources
      //DEBUG_PRINT("      AT pt 7\n");
      return new CIMValue(new UnsignedInt32(0));
   }






   private void setSettingsInResult(CIMInstance ci)
   {
      // copy the settings to the results
      for (int x = 0; x< sv.size(); x++)
      {
         if ("QuickMode".equalsIgnoreCase((String)sv.elementAt(x)))
         {
            if (sQuickMode = false)
               ci.setProperty("QuickMode", new CIMValue(Boolean.FALSE));
            else
               ci.setProperty("QuickMode", new CIMValue(Boolean.TRUE));
         }

         if ("HaltOnError".equalsIgnoreCase((String)sv.elementAt(x)))
         {
            if (sHaltOnError = false)
               ci.setProperty("HaltOnError", new CIMValue(Boolean.FALSE));
            else
               ci.setProperty("HaltOnError", new CIMValue(Boolean.TRUE));
         }

         if ("ReportSoftErrors".equalsIgnoreCase((String)sv.elementAt(x)))
         {
            if (sReportSoftErrors = false)
               ci.setProperty("ReportSoftErrors", new CIMValue(Boolean.FALSE));
            else
               ci.setProperty("ReportSoftErrors", new CIMValue(Boolean.TRUE));
         }

         if ("ReportStatusMessages".equalsIgnoreCase((String)sv.elementAt(x)))
         {
            if (sReportStatusMessages = false)
               ci.setProperty("ReportStatusMessages", new CIMValue(Boolean.FALSE));
            else
               ci.setProperty("ReportStatusMessages", new CIMValue(Boolean.TRUE));
         }

         if ("TestWarningLevel".equalsIgnoreCase((String)sv.elementAt(x)))
            ci.setProperty("TestWarningLevel",
                           new CIMValue(new UnsignedInt16(sTestWarningLevel)));

         if ("PercentOfTestCoverage".equalsIgnoreCase((String)sv.elementAt(x)))
            ci.setProperty("PercentOfTestCoverage",
                           new CIMValue(new UnsignedInt8(sPercentOfTestCoverage)));
      }
   }








   /**
    * The clearResults method will delete the results for the test/MSE combination
    * passed in to this method. 
    * @param op The path to the instance of whose method gets invoked (diag test)
    * @param inParams Vector containing the a REF (CIMObjectPath) for the MSE (device)
    * for which you want the result cleared.
    * @param outParams String array of results that could not be cleared.
    * @param resultclass The name of the result class
    * @return CIMValue return code indicating: (0) method execution was 
    * successful, (1) Unspecified error, (2) Not impelemented or (3) Out of resources.
    *
    * inParams => vector of 1) op to MSE (device) to test
    * outParams => vector of 1) String of result keys that could not be cleared 
    *
    * @throws CIMException
    */
   // method to clear test results
   public CIMValue clearResults(CIMObjectPath op, Vector inParams, Vector outParams,
                                String resultclass, String dtclass)
   throws CIMException
   {
      utils.DEBUG_PRINT(logfile, "      In clearResults method\n");

      IBMSG_DiagResult dr = new IBMSG_DiagResult(resultclass,cimomhandle);

      //$$$ change only the class name not tt
      IBMSG_ServeRAIDTest tt = new IBMSG_ServeRAIDTest();


      // get the test name
      String testname = utils.keyValFromOp(op, "Name");

      int tindex = tt.getTestIndex(testname);

      // get the MSE DeviceID
      // inParams(0) is a CIMValue of the device reference
      CIMValue cv = (CIMValue)inParams.elementAt(0);

      // the value of CIMValue inParams(0) is the CIMObjectPath of the device
      CIMObjectPath dutop = (CIMObjectPath)cv.getValue();

      // the device under test string value
      dut = utils.keyValFromOp(dutop, "DeviceID");

      // build the result op
      CIMObjectPath rop = dr.genResultPath(dtclass, testname, dut);

      // is this result a package result
      CIMValue cvv = cimomhandle.getProperty(rop, "isPackage");

      if (((Boolean)cvv.getValue()).booleanValue() == true)
      {
         Vector pts = new Vector();
         pts = tt.getTestPackage(tindex);

         for (int j = 1; j < pts.size(); j++)
         {
            int tidx = ((UnsignedInt16)pts.elementAt(j)).intValue();
            String testname1 = tt.getTestName(tidx);

            // build the result op
            CIMObjectPath rop1 = dr.genResultPath(dtclass, testname1, dut);

            //delete the result
            dr.deleteDiagResult(rop1);
         }
      }

      //delete the result
      dr.deleteDiagResult(rop);

      // return a CIMValue for the method (NOT the test case)
      // values are: 0 = OK, 1 = Unspecified Error,
      //             2 = Not Implemented, 3 = Out Of Resources
      return new CIMValue(new UnsignedInt32(0));
   }





   //*******************************************************/
   //* private discontinuetest method                      */
   //* inParams => vector of 1) op to MSE (device) to test */
   //*                       2) optional ref to result     */
   //* outParams => vector of 1) Boolean TestingStopped    */
   //*******************************************************/
   // method to abort a test
   public CIMValue discontinueTest(CIMObjectPath op, String settingsclass,
                                   Vector inParms, Vector outParms)
   {
      //utils.DEBUG_PRINT(logfile, "      In discontinuetest method\n");

      // return a CIMValue for the method (NOT the test case)
      // values are: 0 = OK, 1 = Unspecified Error,
      //             2 = Not Implemented, 3 = Out Of Resources
      return  new CIMValue(new UnsignedInt32(2));
   }
}

