/**
 * Title:       RunServeRAIDTest.java
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 * Date:  August 30, 2001 
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/RunServeRAIDTest.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.18
 * Extracted at            : 10:44:44 on 11/12/02
 * Newest delta applied at : 10:35:17 on 4/18/02
 * Notes:
 *  
 * Change History:
 *   Date:			Intials:		Change:
 *   09102001			RHB			-Update provider code to the latest levels
 *   09202001			Rhb			-add changes to the code to allow the retrieveAdapterInfo method to return valid
 *							result objects
 *							-modify runtests & createAdapterInfo to check for the results of the retrieveAdapterInfo method
 *  							-Add new return codes to the resource file....	 
 *   20011311			RHB176539		-Add support for reporting drives that do not support the self-test   
 *   20011411			RHB176539		-Add support for constructing details messages about the unsupported drives when the SCSI self-test
 *							 detects a failure and when it does not detect a failure. 
 *   20020107			RHB178655		-For the status information test add firmware and device location fields. 
 *   20020227			RHB182797		-Changes to adjust the formatting of the details displayed by the consumer.   
 *   20020301			RHB186285		-Broke out tests into individual test classes.  Beefed up command line interface.  		       
 */

//$$$ change package name to provider package name       
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;
import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.text.*;


/**
 * This class is the container for the native method calls (JAVAJ to C++)
 *
 * make sure the dll has the package path
 */

public class RunServeRAIDTest
{
   //Resource Bundle
   public    ResourceBundle  resourceBundle = null;
   public static boolean    DllLoaded      = false;


   //Instance definitions
   public static final int ADAPTER_INFO      = ServeRAID_AdapterInfo.TEST_ADAPTERINFO;
   public static final int ADAPTER_DMA_TEST  = ServeRAID_AdapterDMATest.TEST_ADAPTERDMA;
   public static final int DRIVE_SELF_TEST   = ServeRAID_DriveSelfTest.TEST_DRIVESELF;

   private File  file           = null;
   private PrintStream testLog        = null;

   //This portion of the code will statically load the library associated with the 
   //provided tests.  We also initialize our //errorLog for the test execution at this
   //point and find the ResourceBundle associated with this Locale and desired Format
   static 
   {
      loadDll();
   }

   /**
    * <P> Constructor intializes the resourceBundle to be used for this execution of the test to the default
    *     IBMSG_ServeRAIDBundle.properties. </P>
    */
   public RunServeRAIDTest()
   {
      resourceBundle = ResourceBundle.getBundle("com.ibm.sysmgt.raidmgr.cim.provider.cdm.IBMSG_ServeRAIDBundle",Locale.getDefault());
   }

   /**
    * <P> Constructor intializes the resourceBundle to be used for this execution of the test to a resourceBundle
    *     specified by the creator of this instance. </P>
    *@param resourceBundle<code>ResourceBundle</code><li>Resource bundle to use for this test execution</li>
    */
   public RunServeRAIDTest(ResourceBundle __resourceBundle)
   {
      resourceBundle = __resourceBundle;
   }


   //Static method for loading our library...
   public static void loadDll()
   {
      if (!DllLoaded)
      {
         System.loadLibrary("raid_diags");
         DllLoaded = true;
      }
   }


   /**
    * <P> This method iw used to circumvent the provider code and run the tests from a command line.
    *     Currently this is used for debug purposes only but could be converted into a command line 
    *     executable should the need arise</P>
    */
   public static void main(String args[])
   {
      boolean printHelp    = false;
      int   testSelection  = 0;
      String  identifier   = null;

      System.out.println();
      for (int argNumber = 0; argNumber < args.length; argNumber ++)
      {
         if (argNumber == 0)
         {
            if (args[0].equalsIgnoreCase("help"))
            {
               printHelp = true;
            }
            else
            {
               identifier = (String) args[0];
            }
         }

         else if (argNumber == 1)
         {
            testSelection = Integer.parseInt(args[argNumber]);
            if (testSelection > 3)
            {
               System.out.println("Invalid Parameter:\ttestIndex = " + testSelection 
                                  + "\n\t\t\t"
                                  + "An invalid test was selected. Please select tests 1 through 3 only\n");
               printHelp = true;
            }
         }
      }

      if ((args.length > 0) &&
          !printHelp)
      {
         RunServeRAIDTest runServeRAIDTest = new RunServeRAIDTest();
         Results      result           = new Results();
         if (args.length == 1)
         {
            for (int i = 0; i <= 2; i++)
            {
               runServeRAIDTest.runTest(i,identifier,result);
            }
         }
         else
         {
            runServeRAIDTest.runTest(testSelection - 1,identifier,result);
         }
      }

      if ((args.length <= 0) ||
          (printHelp == true))
      {
         if (args.length <= 0)
         {
            System.out.println("Parameter Mismatch Error:\tNo command line arguments specified.\n");
         }

         System.out.println("Correct syntax for this command:\n\tjava [javaOptions] com.ibm.providers.ServeRAID.RunServeRAIDTest [deviceID] [testIndex]\n");
         System.out.println("\tjava\t\t-Path of java executable");
         System.out.println("\tjavaOptions\t-Options for java command (i.e. classpath...)");
         System.out.println("\tdeviceID\t-PNP device ID of the ServeRAID controller to test");
         System.out.println("\ttestIndex\t-Index of the test to execute");
         System.out.println("\t\t\t\t1 - Adapter Information\n\t\t\t\t2 - Adapter DMA Test\n\t\t\t\t3 - Drive Self Test");
      }
   }




   /**
    * </P>Runs the native serveRAIDTest.  Using this wrapper will allow the 
    * serveRAIDTest to change implementations or names and give the programmer
    * a chance to perform any set-up or clean-up operations.</P>
    *@param<code>int testIndex<code><li>the index of the desired test to Run</li>
    *@param<code>String deviceID<code><li>The device ID retrieved from the CIM repository</li>
    *@param<code>Results results<code><li>Class to hold the test results</li>
    *
    */
   public void runTest(int testIndex, String deviceID, Results result)
   {
      ServeRAIDTest serveRAIDTest = null;
      try
      {
         setDbgParameters();
         if (testIndex == this.ADAPTER_INFO)
         {
            serveRAIDTest = new ServeRAID_AdapterInfo(deviceID,resourceBundle,result);
         }
         else if (testIndex == this.ADAPTER_DMA_TEST)
         {
            serveRAIDTest = new ServeRAID_AdapterDMATest(deviceID,resourceBundle,result);
         }
         else
         {
            serveRAIDTest = new ServeRAID_DriveSelfTest(deviceID,resourceBundle,result);
         }


         serveRAIDTest.setLoggingParameters(testLog, file);
         serveRAIDTest.runServeRAIDTest();
         serveRAIDTest.printResults(result,testLog,file);

      }//end try
      catch (Exception exe)
      {
         result.details.removeAllElements();
         result.errorCode = "035-198-xxx";
         result.details.addElement(resourceBundle.getString("198"));
         if (serveRAIDTest != null)
         {
            serveRAIDTest.printToLog(testLog,file,exe.getLocalizedMessage());
         }
         else
         {
            exe.printStackTrace(testLog);
         }
      }
   }


   /**
    *<P> Sets debugging parameters for this module such as the filename to write data
    *    to and a PrintStream to perform the write operation </P>
    */
   public void setDbgParameters()
   {
      try
      {
         if (file == null)                      //if file has not already been defined
         {
            file = new File(".\\ServeRAIDTestJava.log");       //create a new File object
            if (file.exists() && (testLog == null))          //if this file already exists
            {
               testLog = new PrintStream(new FileOutputStream(file));     //open print Stream
            }
         }
      }
      catch (Exception exe)
      {
      }
   }
}
