/**
 * Title:        ServeRAIDAdapterInfo.java
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 * Date:  August 30, 2001 
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/ServeRAIDAdapterInfo.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.10
 * Extracted at            : 10:44:47 on 11/12/02
 * Newest delta applied at : 15:43:56 on 5/22/02
 * Notes:
 *  
 * Change History:
 *   Date:                      Intials:                Change:
 *   09102001                   RHB                     Update provider code to the latest levels
 */
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;

import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.util.ResourceBundle;
import  java.text.MessageFormat;



/**
 * <P>
 * This class surfaces information about the adapter under test.  The adapter's device
 * driver will query the adapter for the requested information and place the data within
 * this class.  Class RunServeRAIDTest is responsible for requesting the information and 
 * obtaining the information by instantiating this class.</P>
 *@author <code>Rodney H. Brown</code>
 *@version <code>0.1</code>
 */
public class ServeRAIDAdapterInfo
{
    //Resource Bundle to format the drive data
    ResourceBundle resourceBundle;
    Vector adapterInformation       = new Vector(); 
    Vector driveInformation   = new Vector();

    //Define format of adapterInformation and failed drive information

    //Indexes into adapterInformation Vector
    public static final int OFFSET_ADAPTER_STATUS               = 0;
    public static final int OFFSET_ADAPTER_INDEX                = 1;
    public static final int OFFSET_ADAPTER_SLOT                 = 2;
    public static final int OFFSET_ADAPTER_CHANNEL_COUNT        = 3;
    public static final int OFFSET_ADAPTER_DEVICE_COUNT         = 4;
    public static final int OFFSET_BIOS_VERSION                 = 5;
    public static final int OFFSET_FIRMWARE_VERSION             = 6;
    public static final int OFFSET_FRU_NUMBER                   = 7;
    public static final int OFFSET_ADAPTER_MODEL                = 8;
    



    //Adapter Status codes
    private static final String GOOD                            = "0";
    private static final String INVALID_RESPONSE                = "-1";
    private static final String HARDWARE_ERROR                  = "-2";
    private static final String FIRMWARE_ERROR                  = "-3";
    private static final String CHECKSUM_ERROR                  = "-4";
    private static final String SIGNATURE_ERROR                 = "-5";
    private static final String TIMEOUT_ERROR                   = "-6";
    private static final String BIOS_ERROR                      = "-7";
    private static final String COMPARE_ERROR                   = "-8";
    private static final String COMMAND_ERROR                   = "-9";
    private static final String ADAPTER_DEFECTIVE               = "-10";
    private static final String ADAPTER_DISABLED                = "-11";
    private static final String OTHER_STATUS_ERROR              = "-10000";


    /**
     */
    public ServeRAIDAdapterInfo()
    {
    }

    /**
     * Get adapter property
     * @param<code>int propertyIndex</code><li>index of property to retrieve</li>
     * @return<code>Object</code>
     */
    public Object getAdapterProperty(int adapterPropertyIndex)
    {

            //RunServeRAIDTest.errorLog.println("The size of adapter info" + adapterInformation.size());

            if(adapterInformation.isEmpty())
            {
                return null;
            }
            Object[] elements = (Object[])adapterInformation.get(0);
            
            Object returnString = elements[adapterPropertyIndex];
            if(returnString != null)
            {
                return returnString;
            }
            return null;

    }




    /**
     * Return adapter properieties in formatted string
     * @param <code>ResourceBundle</code><li>Resource Bundle to use as format </li>
     * @param <code>boolean addDriveProperties</code><li>If true add drive properties</li>
     *                                               <li>If false exclude drive properties</li>
     * @return <code>String</code><li>Formatted string representation of adapter properties</li>
     */
     public Vector formatAdapterProperties(ResourceBundle resourceBundle, boolean addDriveProperties)
     {

         return formatAdapterProperties(this.adapterInformation, resourceBundle, addDriveProperties);
     }



    /**
     * Return adapter properieties in formatted string
     * @param<code>Vector adapterInfo</code><li>Vector of Object Arrays containing adapter properties</li>
     *@param<code>ResourceBundle</code><li>Resource Bundle containing formats to use for the given strings</li>
     *@param<code>boolean addDriveProperties</code>Set to true to include the drive properties for this adapter...
     * @return <code>Vector</code><li>Formatted Vector of Strings representing the adapter properties</li>
     */
     public Vector formatAdapterProperties(Vector adapterInfo, ResourceBundle resourceBundle, boolean addDriveProperties)
     {
         Vector propertyVector = new Vector();                                                  //Create Vector to return
         
         for(Enumeration enum = adapterInfo.elements();enum.hasMoreElements();)
         {
            Object[] adapterProperties = (Object[])enum.nextElement();                                          //Retrieve the Adapter Information      
             
             //The adapter status will be returned as a single digit code.  The function interpretAdapterStatus
             //will be used to convert the code into a meaningful message for the user.  The resultant message
             //will replace the status code in our adapterProperties variable.
    
             String adapterFamily =MessageFormat.format(resourceBundle.getString("AdapterFamily"),
                                                        adapterProperties);
            adapterFamily += resourceBundle.getString("AdapterFamily."+adapterProperties[this.OFFSET_ADAPTER_MODEL]);
    
            propertyVector.addElement(adapterFamily);
            propertyVector.addElement(MessageFormat.format(resourceBundle.getString("AdapterIndex"),
                                                        adapterProperties));

            propertyVector.addElement(MessageFormat.format(resourceBundle.getString("AdapterDeviceCount"),
                                                        adapterProperties));
             
             String adapterStatus = resourceBundle.getString("AdapterStatus");
             adapterStatus += resourceBundle.getString("AdapterStatus."+adapterProperties[this.OFFSET_ADAPTER_STATUS]);
             propertyVector.addElement(adapterStatus);

             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("AdapterChannelCount"),
                                                        adapterProperties));
             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("AdapterBIOSVersion"),
                                                        adapterProperties));

             //If the caller wishes to add the drive properties then...
             if(addDriveProperties)
             {
                propertyVector.addElement(ServeRAIDHDDiskInfo.formatDriveProperties(this.driveInformation,resourceBundle));
             }
             
         }
            
          return propertyVector;
     }

    /**
     * Returns drive properieties in formatted string
     * @param <code>ResourceBundle resourceBundle</code><li>Properties file(Resource) to use as Format</li>
     * @return <code>Vector</code><li>Formatted string representation of adapter properties</li>
     */
     public Vector formatDriveProperties(ResourceBundle resourceBundle)
     {
        return ServeRAIDHDDiskInfo.formatDriveProperties(this.driveInformation,resourceBundle);
     }




   /**
    * Get adapter information vector from this class
    * @return <code>Vector</code><li>Formatted Adapter information stream</li>
    */
    public Vector getAdapterInfo()
    {
        return adapterInformation;
    }



    /**
     * Add adapter information to the adapterInformation Vector
     * @param<code>Object[] elements</code<li>Object array obtained from raid_diags.dll</li>
     */
    public void addAdapterInfo(Object[] elements)
    {
            adapterInformation.addElement(elements);
    }


   /**
    * Get failed Drive information from this class
    * @return<code>Vector</code><li>Failing disk information</code>
    */
    public Vector getAdapterDriveInfo()
    {
        return driveInformation;
    }

    /**
     * Get disk drive property
     * @param<code>int propertyIndex</code><li>index of property to retrieve</li>
     * @return<code>Object</code>
     */
    public Object getDriveProperty(int driveIndex,int drivePropertyIndex)
    {
        Object[] elements = (Object[])driveInformation.get(driveIndex);
        return (elements[drivePropertyIndex]);
    }

    /**
     * Add drive information to the failedDriveInfo Vector
     * @param<code>Object[] element</code><li>THe array of elements to add</l>
     */
    public void addDriveInfo(Object[] element)
    {
        
        driveInformation.addElement(element);

    }

}


