/**
 * Title:        ServeRAIDHDDiskInfo.java
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 * Date:  August 30, 2001 
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/ServeRAIDHDDiskInfo.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.11
 * Extracted at            : 10:44:49 on 11/12/02
 * Newest delta applied at : 15:43:58 on 5/22/02
 * Notes:
 *  
 * Change History:
 *   Date:                      Intials:                Change:
 *   09102001                   RHB                     Update provider code to the latest levels
 */
 
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;
import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.text.*;


/**
 * This storage class contains information pertaining to a single hard drive.  The fields
 * in the class will be provided by the raid_diags.dll function getAdapterInfo() and will
 * be used as a vehicle to pass information about drives experiencing PFA events or drives
 * in a DDD state.
 *
 *@author<code>Rodney H. Brown</code>
 *@version<code>v0.1</code>
 */
public class ServeRAIDHDDiskInfo
{


    //Indexes into failedDriveInformation Vector
    public static final int OFFSET_SCSI_ID                      = 0;
    public static final int OFFSET_LUN_ID                       = 2;
    public static final int OFFSET_TARGET_ID                    = 1;
    public static final int OFFSET_PRODUCT_ID                   = 3;
    public static final int OFFSET_VENDOR_ID                    = 4;
    public static final int OFFSET_PRODUCT_REVISION             = 5;
    public static final int OFFSET_DRIVE_FRUNUMBER              = 6;
    public static final int OFFSET_DRIVE_STATE                  = 7;
    public static final int OFFSET_PFA_STATUS                   = 8;
    public static final int OFFSET_SELFTEST_RESULTS             = 9;

    //Drive status codes
    public static final String BAD_STATE                        = "0";
    public static final String EMP_STATE                        = "1";
    public static final String DHS_STATE                        = "2";
    public static final String DDD_STATE                        = "3";
    public static final String SBY_STATE                        = "4";
    public static final String RDY_STATE                        = "5";
    public static final String HSP_STATE                        = "6";
    public static final String ONL_STATE                        = "7";
    public static final String RBL_STATE                        = "8";
    public static final String SHS_STATE                        = "9";

    //PFA status
    public static final String PFA                              = "1";
    public static final String NO_PFA                           = "0";

    //Self Test status
    public static final String  PASSED                          = "0";
    public static final String  FAILED                          = "1";
    public static final String  NOT_SUPPORTED                   = "2";
    public static final String  BUSY                            = "3";

    /**
     * Returns drive properieties in formatted string
     * @param<code> Gets a drive Property <code>
     * @return <code>String</code><li>Formatted string representation of adapter properties</li>
     */
    public static String getDriveProperty(Object[] driveProperties, int propertyIndex)
    {
        try
        {
            String returnString =  (String)driveProperties[propertyIndex];
            return returnString;
        }
        catch(Exception exception)
        {
            return new String();
        }
        
    }

        /**
     * Returns drive properieties in formatted string
     * @return <code>String</code><li>Formatted string representation of adapter properties</li>
     */
     public static Vector formatDriveProperties(Vector driveInformation,ResourceBundle resourceBundle)
     {
         return formatDriveProperties(driveInformation,resourceBundle,false);
     }

    /**
     * Returns drive properieties in formatted string
     * @return <code>String</code><li>Formatted string representation of adapter properties</li>
     */
     public static Vector formatDriveProperties(Vector driveInformation,ResourceBundle resourceBundle, boolean addSelfTestResults)
     {
         Vector propertyVector = new Vector();

         for(Enumeration enum = driveInformation.elements(); enum.hasMoreElements();)
         {
             
             Object[] driveProperties = (Object[])enum.nextElement();
    
             String driveStatus = resourceBundle.getString("DriveStatus");
             driveStatus += MessageFormat.format(resourceBundle.getString("DriveStatus."+driveProperties[OFFSET_DRIVE_STATE]),
                                                        driveProperties);
             propertyVector.addElement(driveStatus);
             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("DriveLocation"),
                                                        driveProperties));
             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("Manufacturer"),
                                                        driveProperties));
             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("Product"),
                                                        driveProperties));
             propertyVector.addElement(MessageFormat.format(resourceBundle.getString("ProductRevision"),
                                                        driveProperties));
    
             String fruNumber = getDriveProperty(driveProperties,OFFSET_DRIVE_FRUNUMBER);
             if(!fruNumber.equals("unknown"))
             {
                propertyVector.addElement(MessageFormat.format(resourceBundle.getString("FRU"),
                                                        driveProperties));
             }
             else
             {
                 propertyVector.addElement(MessageFormat.format(resourceBundle.getString("FRU_Unknown"),
                                                        driveProperties));
             }

             propertyVector.addElement(resourceBundle.getString("PFAEvent") + " " 
                                       + resourceBundle.getString("pfaAlert."
                                       +driveProperties[OFFSET_PFA_STATUS]));
            
             if(true == addSelfTestResults)
             {
                 
                 propertyVector.addElement(resourceBundle.getString("SelfTestResult") + " " 
                                           + resourceBundle.getString("selfTestResult."
                                           +driveProperties[OFFSET_SELFTEST_RESULTS]));
             }

             propertyVector.addElement(" ");

         }
         
         
         return propertyVector;
     }



}
