/**
 * Description:  ServeRAIDTest parent class
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 *
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/ServeRAIDTest.java, sv_esw_cdm_provider, raid_prov, 20021101
 * @version                : 1.1
 * Extracted at            : 10:44:50 on 11/12/02
 * Newest delta applied at : 15:17:14 on 4/2/02
 * Notes:
 *  
 * Change History:
 *   Date:			Intials:		Change:
 *			       
 */

//$$$ change package name to provider package name       
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;
import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.text.*;


/**
 * <P> ServeRAIDTest is the parent class for all test objects.   ServeRAID test objects must subclass from this object.  The class defines
 *     the common behaviour and characteristics of a ServeRAIDTest.  Any of the methods defined in this class may be overridden to customize
 *	the behaviour of a test. </P>
 *
 *	<P>Two native methods are defined:
 *	<ol>
 *		<li>serveRAIDTest</li>
 *		<li>retrieveAdapterInfo</li>
 *	</ol></P>
 *	<P> These methods access functions within raid_diags.dll </P>
 */
public class ServeRAIDTest 
{

   protected Results testResult = null;                            //Result of test
   protected Results adapterInfoResult = null;                     //Result of adapter information query
   protected ServeRAIDAdapterInfo serveRAIDAdapterInfo = null;     //adapter information retrieved
   protected String deviceID = null;                               //ID of the device under test
   protected int testIndex= 0;                                     //index of the test under execution
   protected ResourceBundle resourceBundle = null;                 //ResourceBundle to use for this test
   protected Vector driveInfo = null;                              //Adapter physical drive information
   protected File file = null;
   protected PrintStream testLog = null;     


   //Native method declarations
   protected native void serveRAIDTest(int testIndex, String deviceID, Results result);
   protected native void retrieveAdapterInfo(String deviceID, ServeRAIDAdapterInfo serveRAIDAdapterInfo);


   /**
    * <P> Default constructor </P>
    */
   public ServeRAIDTest()
   {
   }


   /**
    * <P> Common constructor for all ServeRAIDTest children.  This constructor will initialize to adapter information
    *     query and save the results in the serveRAIDAdapterInformation field. </P>
    * @param _deviceID <code>String</code><li>Device ID of the device under test</li>
    * @param _resourceBundle <code>ResourceBundle</code><li>external file containing the valid string definitions for this test.</li>
    *
    */
   public ServeRAIDTest(String _deviceID,ResourceBundle _resourceBundle,Results _result) throws Exception
   {

      deviceID    = _deviceID;
      resourceBundle    = _resourceBundle;
      testResult  = _result;
      driveInfo   = new Vector();


      try
      {
         adapterInfoResult     = new Results();         //result of our inquiry (pass/fail/other, errorCode, details)
         serveRAIDAdapterInfo = new ServeRAIDAdapterInfo();     //Object to contain information gathered

         this.retrieveAdapterInfo(deviceID,serveRAIDAdapterInfo);  //Retrieve information about device
         if (adapterInfoResult.statusCode == Results.RESULT_PASSED)
         {

            adapterInfoResult.deviceLocation     = MessageFormat.format(resourceBundle.getString("AdapterLocation"),
                                                                        new Object[]{serveRAIDAdapterInfo.getAdapterProperty(serveRAIDAdapterInfo.OFFSET_ADAPTER_SLOT)});

            adapterInfoResult.firmwareLevel     = (String) serveRAIDAdapterInfo.getAdapterProperty(
                                                                                                  serveRAIDAdapterInfo.OFFSET_FIRMWARE_VERSION);
         }

         testResult.statusCode       = adapterInfoResult.statusCode;
         testResult.deviceLocation      = adapterInfoResult.deviceLocation;
         testResult.details       = adapterInfoResult.details;
         testResult.errorCode     = adapterInfoResult.errorCode;
         testResult.firmwareLevel       = adapterInfoResult.firmwareLevel;
         printToLog(testLog,file,"Status code " + testResult.statusCode + "\nFirmwareLevel" + testResult.firmwareLevel);

      }
      catch (Exception anyException)
      {
         testResult.statusCode = testResult.RESULT_FAILED;
         printToLog(testLog,file,anyException.getLocalizedMessage());
         throw anyException;
      }
   }


   /**
    * <P> Returns the adapter information for this device </P>
    * @return <code>ServeRAIDAdapterInfo</code>
    */
   public ServeRAIDAdapterInfo getAdapterInfo()
   {
      return this.serveRAIDAdapterInfo;
   }


   /**
    * <P> Returns the result of the adapter information query </P>
    *@return <code>Results</code>
    */

   public Results getAdapterInfoResults()
   {
      return this.adapterInfoResult;
   }


   /**
    * <P> Runs the designated test, and executes the appropriate details and actionONFailure
    *	   method for the test under execution.  Child classes may override this class in order 
    *     to implement a different testing algorithm. </P>
    */
   public void runServeRAIDTest()
   {
      try
      {
         //Run the test
         serveRAIDTest(testIndex,deviceID,testResult);

         if (testResult.statusCode != testResult.RESULT_STOPPED)
         {
            //Generate the test details
            getTestDetails();

            if (testResult.statusCode == testResult.RESULT_FAILED)
            {
               //Generate actions if there is a failure
               getActionOnFailure();
            }
         }

         //Generate the error code Text description...
         if (testResult.errorCode.length() >= 7 )
         {
            String functionCode = testResult.errorCode.substring(4,7);
            if (testResult.details.isEmpty())
            {
               testResult.details.addElement(resourceBundle.getString(functionCode));
            }
            else
            {
               testResult.details.insertElementAt(resourceBundle.getString(functionCode),0);
            }
         }
      }
      catch (MissingResourceException missingResource)
      {
         testResult.details.removeAllElements();
         testResult.errorCode = "035-198-xxx";
         testResult.details.addElement("Unable to display results for this test"); 
         printToLog(testLog,file,(String) testResult.details.lastElement());
         testLog.close();
      }
      catch (Exception anyException)
      {
         testResult.details.removeAllElements();
         testResult.errorCode = "035-198-xxx";
         testResult.details.addElement(resourceBundle.getString("198"));
         printToLog(testLog,file,anyException.getLocalizedMessage());
         testLog.close();
      }
   }


   /**
    * <P>Returns the results of this test execution.</P>
    */
   public Results getTestResults()
   {
      return this.testResult;
   }


   /**
    *<P> Gets the details of the last test execution.</P>
    */
   public Vector getTestDetails() throws MissingResourceException, Exception
   {
      return testResult.details = new Vector();
   }


   /**
    *<P> Gets the actions for the failures detected by the last test execution.</P>
    */
   public Vector getActionOnFailure() throws MissingResourceException, Exception
   {
      return testResult.actionOnFailure = new Vector();
   }


   /**
    *<P> Method to allow the native code to update this class with drive information.</P>
    */
   public void addDriveInformation(Object[] elements)
   {
      driveInfo.addElement(elements);
   }

   /**
    * <P>Returns the drive information for this adapter<P>
    */
   public Vector getDriveInformation()
   {
      return driveInfo;
   }

   /**
    * <p> Set the PrintStream and file parameters<p>
    */
   public void setLoggingParameters(PrintStream _testLog, File _file)
   {
      testLog = _testLog;
      file  = _file;
   }


   /**
    * Print to printStream
    * @param message <code>String</code><li>Message to print to stream</li>
    * @return byte <li> length of the message written to the stream</li>
    */
   public int printToLog(String message)
   {
      return printToLog(testLog,file,message);
   }

   /**
    * Print to printStream
    * @param testLog <code>PrintStream</code><li>Stream to print to (i.e Standard output, file...)</li>
    * @param file <code>File</code><li>Check for file to determine whether or not to print to screen </li>
    * @param message <code>String</code><li>Message to print to stream</li>
    * @return byte <li> length of the message written to the stream</li>
    */
   public int printToLog(PrintStream testLog, File file, String message)
   {
      if (file != null)
      {

         if (file.exists())
         {
            testLog.println(message);
            return message.length();
         }
         else
         {
            return 0;
         }
      }
      return 0;

   }

   /**
    * Print all fields in the results object
    * @param result <code>Results</code><li>Result object to print</li>
    */
   public void printResults(Results result, PrintStream testLog, File file)
   {
      printToLog(testLog,file,"\n\nResults for execution of test " + testIndex);
      printToLog(testLog,file,"Device Location:\t" + result.deviceLocation);
      printToLog(testLog,file,"Error Code:\t" + result.errorCode);
      printToLog(testLog,file,"Firmware Level:\t" + result.firmwareLevel);

      if (!result.details.isEmpty())
      {
         printToLog(testLog,file,"\nDetails of this test Execution :\n");
         for (Enumeration enum = result.details.elements(); enum.hasMoreElements();)
         {
            printToLog(testLog,file,(String) enum.nextElement());
         }
      }

      if (!result.actionOnFailure.isEmpty())
      {
         printToLog(testLog,file,"\nActions to to take as a result of this test:\n");
         for (Enumeration enum = result.actionOnFailure.elements(); enum.hasMoreElements();)
         {
            printToLog(testLog,file,(String) enum.nextElement());
         }
      }
      printToLog(testLog,file,"\n\n");
   }
}
