/**
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 *
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/ServeRAID_AdapterInfo.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.3
 * Extracted at            : 10:44:51 on 11/12/02
 * Newest delta applied at : 15:43:53 on 5/22/02
 * Notes:
 *  
 * Change History:
 *   Date:                      Intials:                Change:
 *                             
 */

//$$$ change package name to provider package name       
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;
import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.text.*;


/**
 * <P> Implementation of the Adapter Inforamtion module subclasses from ServeRAIDTest </P>
 */
public class ServeRAID_AdapterInfo extends ServeRAIDTest
{
   public static final int TEST_ADAPTERINFO = 0;

   private     Vector criticalDrives   = new Vector();
   private     String drivePfa         = new String();
   private     String driveState       = new String();

   /**
    *<P>Default constructor.</P>
    */
   public ServeRAID_AdapterInfo(String _deviceID, ResourceBundle resourceBundle, Results result) throws Exception
   {
      super(_deviceID, resourceBundle, result);
      testIndex = TEST_ADAPTERINFO;
   }


   /**
    * <P> Overides the runServeRAIDTest method of ServeRAIDTest.  This method
    *     displays vital information about the adapter. </P>
    */
   public void runServeRAIDTest() 
   {
      try
      {
         if (adapterInfoResult.statusCode == testResult.RESULT_PASSED)
         {
            //Generate the test details
            getTestDetails();

            //Generate actions if there is a failure
            getActionOnFailure();

         }

         //Generate the error code Text description...
         if ((adapterInfoResult.errorCode.length() >= 7 ) &&
             (adapterInfoResult.statusCode != testResult.RESULT_PASSED))
         {
            String functionCode = testResult.errorCode.substring(4,7);
            if (testResult.details.isEmpty())
            {
               testResult.details.addElement(resourceBundle.getString(functionCode));
            }
            else
            {
               testResult.details.insertElementAt(resourceBundle.getString(functionCode),0);
            }
         }
      }
      catch (MissingResourceException resourceMissing)
      {
         testResult.actionOnFailure.addElement("Unable to display error code for the indicated failure");
         printToLog(testLog,file,resourceMissing.getLocalizedMessage());
      }
      catch (Exception anyException)
      {
         testResult.details.removeAllElements();
         testResult.errorCode = "035-198-xxx";
         testResult.details.addElement(resourceBundle.getString("198"));
         printToLog(testLog,file,anyException.getLocalizedMessage());
         anyException.printStackTrace(testLog);
      }
   }


   /**
    *<P> Gets the details of the last test execution.</P>
    * @throws MissingResourceException
    * @throws Exception     
    */
   public Vector getTestDetails() throws MissingResourceException, Exception
   {
      try
      {
         Object[] adapterFirmware = {adapterInfoResult.firmwareLevel};

         //Ensure that if this test is designed to be a viewer the device location and
         //firmware levels of the device are appropriately displayed.
         testResult.details.addElement(resourceBundle.getString("DeviceLocation") +
                                       adapterInfoResult.deviceLocation);

         for (Enumeration adapterPropEnum = serveRAIDAdapterInfo.formatAdapterProperties(resourceBundle,false).elements();
             adapterPropEnum.hasMoreElements();)
         {
            testResult.details.addElement(adapterPropEnum.nextElement());
         }

         testResult.details.addElement(MessageFormat.format(resourceBundle.getString("AdapterFirmwareVersion"),
                                                            adapterFirmware));

         for (Enumeration driveEnum = serveRAIDAdapterInfo.getAdapterDriveInfo().elements(); driveEnum.hasMoreElements();)
         {
            Object[] drive = (Object []) driveEnum.nextElement();
            drivePfa = ServeRAIDHDDiskInfo.getDriveProperty(drive,ServeRAIDHDDiskInfo.OFFSET_PFA_STATUS);
            driveState = ServeRAIDHDDiskInfo.getDriveProperty(drive,ServeRAIDHDDiskInfo.OFFSET_DRIVE_STATE);

            if (ServeRAIDHDDiskInfo.PFA.equals(drivePfa)
                || ServeRAIDHDDiskInfo.DDD_STATE.equals(driveState)
                || ServeRAIDHDDiskInfo.DHS_STATE.equals(driveState)
                || ServeRAIDHDDiskInfo.BAD_STATE.equals(driveState))
            {
               criticalDrives.addElement(drive);
            }
         }

         if (!criticalDrives.isEmpty())
         {
            testResult.details.addElement(" ");
            testResult.details.addElement( resourceBundle.getString("PFADrives"));
            testResult.details.addElement(" ");
            testResult.details.addAll(ServeRAIDHDDiskInfo.formatDriveProperties(criticalDrives,resourceBundle));
         }
      }
      catch (MissingResourceException resourceMissing)
      {
         throw resourceMissing;
      }
      catch (Exception anyException)
      {
         throw anyException;
      }

      return testResult.details;
   }


   /**
    *<P> Gets the actions for the failures detected by the last test execution.</P>
    * @throws MissingResourceException
    * @throws Exception     
    */
   public Vector getActionOnFailure() throws MissingResourceException, Exception
   {
      try
      {

         if (!criticalDrives.isEmpty())
         {
            if (ServeRAIDHDDiskInfo.PFA.equals(drivePfa)
                || ServeRAIDHDDiskInfo.DDD_STATE.equals(driveState)
                || ServeRAIDHDDiskInfo.DHS_STATE.equals(driveState)
                || ServeRAIDHDDiskInfo.BAD_STATE.equals(driveState))
            {
               testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.adapter.check_cables"));
            }

            testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.drive.check_firmware_level"));
            testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.drive.reboot_system"));
            testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.call_service"));
         }
      }
      catch (MissingResourceException resourceMissing)
      {
         throw resourceMissing;
      }
      catch (Exception anyException)
      {
         throw anyException;
      }
      return testResult.actionOnFailure;
   }
}
