/**
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      IBM
 *
 *
 * @author Rodney H. Brown
 * Path, component, release: prov/com/ibm/providers/ServeRAID/ServeRAID_DriveSelfTest.java, sv_esw_cdm_provider, raid_prov, 20021101
 * Version                 : 1.3
 * Extracted at            : 10:44:53 on 11/12/02
 * Newest delta applied at : 15:43:55 on 5/22/02
 * Notes:
 *  
 * Change History:
 *   Date:                      Intials:                Change:
 *                             
 */

//$$$ change package name to provider package name       
package com.ibm.sysmgt.raidmgr.cim.provider.cdm;
import  java.io.*;
import  java.lang.*;
import  java.util.*;
import  java.text.*;




/**
 * <P> Implementation of the Drive Self Test subclasses from ServeRAIDTest </P>
 */

public class ServeRAID_DriveSelfTest extends ServeRAIDTest
{
    public static final int TEST_DRIVESELF = 2;


    /**
     *<P>Default constructor.</P>
     */
    public ServeRAID_DriveSelfTest(String _deviceID, ResourceBundle resourceBundle, Results result) throws Exception
    {
        super(_deviceID, resourceBundle, result);
        testIndex = TEST_DRIVESELF;
    }


    /**
     *<P> Gets the details of the last test execution.</P>
     * @throws MissingResourceException
     * @throws Exception     
     */
    public Vector getTestDetails() throws MissingResourceException, Exception
    {
        try
        {
    
            if(testResult.statusCode != testResult.RESULT_STOPPED)
            {

                //If any drives failed self-test print out the appropriate information
                if(!driveInfo.isEmpty())
                {
                    testResult.details.addElement(" ");

                    for(Enumeration propertyEnum = ServeRAIDHDDiskInfo.formatDriveProperties(driveInfo,resourceBundle,true).elements();
                        propertyEnum.hasMoreElements();)
                    {
                        testResult.details.addElement(propertyEnum.nextElement());
                    }

                }
            }//end if testResult.statusCode == testResult.FAILED
        }
        catch(MissingResourceException resourceMissing)
        {
            throw resourceMissing;
        }
        catch(Exception anyException)
        {
            throw anyException;
        }

        return testResult.details;
    }


    
    /**
     *<P> Gets the actions for the failures detected by the last test execution.</P>
     * @throws MissingResourceException
     * @throws Exception     
     */
    public Vector getActionOnFailure() throws MissingResourceException, Exception
    {
        try
        {
            boolean driveBusy = false;
            for(Enumeration driveEnum = driveInfo.elements();driveEnum.hasMoreElements();)
            {
                String driveTestResult = (String)(ServeRAIDHDDiskInfo.getDriveProperty((Object[])driveEnum.nextElement(),
                                                                                       ServeRAIDHDDiskInfo.OFFSET_SELFTEST_RESULTS));
                if(ServeRAIDHDDiskInfo.BUSY.equals(driveTestResult))
                {
                    driveBusy = true;
                }
            }
            
            if(driveBusy)
            {
                testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.drive.Busy"));
            }
            if(testResult.statusCode== testResult.RESULT_FAILED)
            {
                testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.adapter.check_cables"));
                testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.drive.check_firmware_level"));
                testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.drive.reboot_system"));
                testResult.actionOnFailure.addElement(resourceBundle.getString("ActionItem.call_service"));
            }

        }
        catch(MissingResourceException resourceMissingException)
        {
            throw resourceMissingException;
        }
        catch(Exception anyException)
        {
            throw anyException;
        }


        return testResult.actionOnFailure;
    }
}
