package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class RAIDArrayProvider extends RAIDInstanceProviderBase
{
   public RAIDArrayProvider()
   {
      super("RAIDArrayProvider:");
      instanceLog.println(logHeader + "RAIDArrayProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(BasicArray.CIM_CREATION_CLASS_NAME))
         instances = enumRAIDArrays(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      

      return cops;
   }


   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(BasicArray.CIM_CREATION_CLASS_NAME))
      {         
         
         return enumRAIDArrays(system, cc);
      }         
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");      

      

      RaidSystem system = null;
      try
      {
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(BasicArray.CIM_CREATION_CLASS_NAME))
      {
         
         return getRAIDArray(system, cc, op);
      }         
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // RAID Arrays
   // **********************
   public Vector enumRAIDArrays(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {         
         Adapter adapter = (Adapter)e.nextElement();

         // Enumerates only non-sub-arrays
         for (Enumeration e2 = adapter.enumerateBasicArrays(); e2.hasMoreElements();)
         {            
            BasicArray array = (BasicArray)e2.nextElement();
            instances.addElement(array.createCIMInstance(cc.newInstance()));
         }         

         // Get the sub-arrays from the spanned arrays
         for (Enumeration e2 = adapter.enumerateSpannedArrays(); e2.hasMoreElements();)
         {
            SpannedArray spannedArray = (SpannedArray)e2.nextElement();
            for (Enumeration e3 = spannedArray.enumerateArrays(); e3.hasMoreElements();)
            {
               BasicArray array = (BasicArray)e3.nextElement();
               instances.addElement(array.createCIMInstance(cc.newInstance()));
            }
         }
      }
      
      return instances;
   }   

   public CIMInstance getRAIDArray(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String collectionID = (String)getKeyValue(op, "CollectionID");

         int controllerID = Adapter.displayIDToAdapterID(getToken(collectionID, ":", "ControllerID"));
         int arrayID = BasicArray.displayIDToArrayID(getToken(collectionID, ":", "ArrayID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);

         // Both spanned arrays and basic arrays can have the same array ID, so try getting the
         // sub array first so that you don't get a spanned array by mistake.
         if (adapter.getSubArray(arrayID) != null)
            return adapter.getSubArray(arrayID).createCIMInstance(cc.newInstance());
         else
            return adapter.getArray(arrayID).createCIMInstance(cc.newInstance());                 
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }


    public static void main(String [] args)
    {
        RAIDArrayProvider provider = new RAIDArrayProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
