package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class SCSITapeDriveProvider extends RAIDInstanceProviderBase
{
   public SCSITapeDriveProvider()
   {
      super("SCSITapeDriveProvider:");
      instanceLog.println(logHeader + "SCSITapeDriveProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(TapeDrive.CIM_CREATION_CLASS_NAME))
         instances = enumSCSITapeDrives(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      
      return cops;
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(TapeDrive.CIM_CREATION_CLASS_NAME))
      {
         
         return enumSCSITapeDrives(system, cc);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(TapeDrive.CIM_CREATION_CLASS_NAME))
      {
         
         return getSCSITapeDrive(system, cc, op);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // SCSI CD-ROM Drives
   // **********************
   public Vector enumSCSITapeDrives(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.TAPE)).elements(); e.hasMoreElements();)
      {
         TapeDrive tape = (TapeDrive)e.nextElement();
         instances.addElement(tape.createCIMInstance(cc.newInstance()));
      }

      return instances;
   }   

   public CIMInstance getSCSITapeDrive(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int tapeID = Integer.parseInt(getToken(deviceID, ":", "TapeDriveID"));         
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);

         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, tapeID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         TapeDrive tape = (TapeDrive) devices.elementAt(0);
         return tape.createCIMInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }


    public static void main(String [] args)
    {
        SCSITapeDriveProvider provider = new SCSITapeDriveProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
