package com.ibm.sysmgt.raidmgr.cim.provider.v2;

import java.util.*;
import javax.wbem.cim.*;
import javax.wbem.client.*;

public class ProviderUtil
{
   public static final String CIM_WINDOWS_SYSTEM_CREATION_CLASS_NAME = "Win32_ComputerSystem";
   public static final String CIM_LINUX_SYSTEM_CREATION_CLASS_NAME = "IBMPSG_ComputerSystem";
   public static CIMDataType UINT16_ARRAY_TYPE = new CIMDataType(CIMDataType.UINT16_ARRAY);

   /** Retrieves a key of from the given CIMObjectPath
     * @param op The CIMObjectPath with appropiate keys
     * @return The value of the key, or null if the key is not found
     */
   public static Object getKeyValue(CIMObjectPath op, String keyName)
   {
      Vector keys = op.getKeys();

      for (Enumeration e = keys.elements(); e.hasMoreElements();)
      {
         CIMProperty key = (CIMProperty)e.nextElement();
         if (key.getName().equals(keyName))
            return key.getValue().getValue();            
      }

      return null;
   }

   /** Parses a string for a key value
     *
     *   Example:
     *   str = "AdapterID:1:ChannelID:2:TargetID:14"
     *   delim = ":"
     *   key = "ChannelID"
     *
     *   getToken will return "2"
     * 
     * @param op The CIMObjectPath with appropiate keys
     * @return The value of the key, or null if the key is not found
     */
   public static String getToken(String str, String delim, String key)
   {
      StringTokenizer t = new StringTokenizer(str, delim);

      while (t.hasMoreTokens())
      {
         if (t.nextToken().equals(key))
            return t.nextToken();
      }

      return null;
   }

   public static String getSystemCreationClassName()
   {
      return "CIM_ComputerSystem";
      /*
      if (JCRMOS.getOS() == JCRMOS.WINDOWS)
         return CIM_WINDOWS_SYSTEM_CREATION_CLASS_NAME;
      else if ((JCRMOS.getOS() == JCRMOS.LINUX))
         return CIM_LINUX_SYSTEM_CREATION_CLASS_NAME;
      else
         return "Unknown";
      */   
   }

   /**
     * @return TRUE if the network is enabled; FALSE otherwise. 
     */
   public static boolean isNetworkEnabled()
   {      
      try
      {
         if (java.net.InetAddress.getLocalHost() != null)
            return true;
         else
            return false;
      }
      catch (Exception ex)
      {
         return false;
      }      
   }

   public static CIMInstance getInstanceFromRefArgument(ProviderCIMOMHandle cimom, CIMArgument arg) throws CIMException
   {      
      try
      {
         CIMObjectPath op = (CIMObjectPath)arg.getValue().getValue();      
         return cimom.getInstance(op, true, false, false, null);
      }      
      catch (ClassCastException ex)
      {         
         RAIDProvider.log.println("!!! (ProviderUtil.getInstanceFromRefArgument) argument is not REF type.");
         new Throwable().printStackTrace(RAIDProvider.log);
         throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER);
      }
   }
}
