
package com.ibm.sysmgt.raidmgr.cim.provider.v2.association;

import com.ibm.sysmgt.raidmgr.cim.provider.v2.instance.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.ProviderUtil;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import javax.wbem.cim.*;

import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.util.*;

public class AssociatorDiskDrive extends AssociatorProviderBase
{
   public AssociatorDiskDrive()
   {
      super("AssociatorDiskDrive:", "ADPT_RAIDRealizesPhysicalDisk");
      associationLog.println(logHeader + "AssociatorDiskDrive()");
   }

   protected Vector associatorHelper(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException
   {
      if (objectName.getObjectName().equalsIgnoreCase(RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME /* IBMPSG_RAIDDiskDrive */))
      {
         associationLog.println(logHeader + "(associatorHelper) Getting associators for " + RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME);

         Vector instances = new Vector();

         // IBMPSG_RAIDDiskDrive plays the ANTECEDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(ANTECEDENT)))
         {
            associationLog.println("!!!" + logHeader + RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }
         
         CIMObjectPath tempOP = new CIMObjectPath(RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME);
         tempOP.setNameSpace(assocName.getNameSpace());
         CIMClass cc = cimom.getClass(tempOP, false, false, false, null);
         
         String deviceID = (String)ProviderUtil.getKeyValue(objectName, "DeviceID");
         int diskDriveID = Integer.parseInt(ProviderUtil.getToken(deviceID, ":", "DiskDriveID"));
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(deviceID, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {         
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
            {
               CIMInstance driveInstance = RAIDDiskDriveProvider.createCIMPhysicalDiskDriveInstance(drive, cc.newInstance());
               instances.addElement(driveInstance);
            }
         }

         associationLog.println(logHeader + "associatorHelper returning " + instances.size() + " instances.");
         return instances;
      }
      else if (objectName.getObjectName().equalsIgnoreCase(RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME /* IBMPSG_PhysicalDiskDrive */))
      {
         Vector instances = new Vector();

         associationLog.println(logHeader + "(associatorHelper) Getting associatorNames for " + RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME);

         // IBMPSG_PhysicalDiskDrive plays the DEPENDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(DEPENDENT)))
         {
            associationLog.println("!!!" + logHeader + RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }
         
         CIMObjectPath tempOP = new CIMObjectPath(RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME);
         tempOP.setNameSpace(assocName.getNameSpace());
         CIMClass cc = cimom.getClass(tempOP, false, false, false, null);
         
         String tag = (String)ProviderUtil.getKeyValue(objectName, "Tag");
         int diskDriveID = Integer.parseInt(ProviderUtil.getToken(tag, ":", "DeviceID"));
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(tag, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {         
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
            {
               CIMInstance driveInstance = RAIDDiskDriveProvider.createCIMInstance(drive, cc.newInstance());
               instances.addElement(driveInstance);
            }
         }

         associationLog.println(logHeader + "associatorHelper returning " + instances.size() + " instances.");
         return instances;
      }

      associationLog.println("!!!" + logHeader + "associatorHelper found no matches, throwing exception.");

      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   protected Vector referenceHelper(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException
   {
      if (objectName.getObjectName().equalsIgnoreCase(RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME /* IBMPSG_RAIDDiskDrive */))
      {
         Vector instances = new Vector();
      
         associationLog.println(logHeader + "(referenceHelper) Getting referenceNames for " + RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME);         

         // IBMPSG_RAIDDiskDrive plays the ANTECEDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(ANTECEDENT)))             
         {
            associationLog.println("!!!" + logHeader + RAIDDiskDriveProvider.CIM_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }

         CIMClass cc = cimom.getClass(assocName, false, false, false, null);

         String deviceID = (String)ProviderUtil.getKeyValue(objectName, "DeviceID");
         int diskDriveID = Integer.parseInt(ProviderUtil.getToken(deviceID, ":", "DiskDriveID"));
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(deviceID, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {         
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
            {
               CIMInstance driveInstance = createRealizesPhysicalDiskAssociation(drive, cc.newInstance());
               instances.addElement(driveInstance);
            }
         }

         associationLog.println(logHeader + "referenceHelper returning " + instances.size() + " instances.");

         return instances;
      }
      else if (objectName.getObjectName().equalsIgnoreCase(RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME /* IBMPSG_PhysicalDiskDrive */))
      {
         Vector instances = new Vector();
      
         associationLog.println(logHeader + "(referenceHelper) Getting referenceNames for " + RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME);

         // IBMPSG_PhysicalDiskDrive plays the DEPENDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(DEPENDENT)))             
         {
            associationLog.println("!!!" + logHeader + RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }         

         CIMClass cc = cimom.getClass(assocName, false, false, false, null);

         String tag = (String)ProviderUtil.getKeyValue(objectName, "Tag");
         int diskDriveID = Integer.parseInt(ProviderUtil.getToken(tag, ":", "DeviceID"));
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(tag, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {         
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
            {
               CIMInstance driveInstance =createRealizesPhysicalDiskAssociation(drive, cc.newInstance());
               instances.addElement(driveInstance);
            }
         }

         associationLog.println(logHeader + "referenceHelper returning " + instances.size() + " instances.");

         return instances;
      }

      associationLog.println("!!!" + logHeader + "referenceNames found no matches, throwing exception.");

      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }


   //**************************************************************************************************************
   // Now for the InstanceProvider implementation (for Windows)
   //**************************************************************************************************************

   public CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      associationLog.println(logHeader + "op = " + op);
      associationLog.println(logHeader + "op.getObjectName() = " + op.getObjectName());
      associationLog.println(logHeader + "CIM_CREATION_CLASS_NAME = " + CIM_CREATION_CLASS_NAME);
      associationLog.println(logHeader + "op.getObjectName().length() = " + op.getObjectName().length());
      associationLog.println(logHeader + "CIM_CREATION_CLASS_NAME.length() = " + CIM_CREATION_CLASS_NAME.length());


      if (!op.getObjectName().trim().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
      
      Vector instances = new Vector();

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(createRealizesPhysicalDiskAssociation(drive, cc.newInstance()));
         }         
      }      

      return (CIMInstance[])instances.toArray(new CIMInstance[instances.size()]);      
   }

   public CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide names of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      

      Vector cops = new Vector();
      Vector instances = new Vector();

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(createRealizesPhysicalDiskAssociation(drive, cc.newInstance()));
         }         
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }
      
      return (CIMObjectPath[])cops.toArray(new CIMObjectPath[cops.size()]);      
   }

   public CIMInstance createRealizesPhysicalDiskAssociation(HardDrive drive, CIMInstance instance)
   {
      // Antecedent
      instance.setProperty("Antecedent", new CIMValue(RAIDDiskDriveProvider.getCIMObjectPath(drive)));

      // Dependent
      CIMObjectPath op2 = new CIMObjectPath(RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME);

      op2.addKey("CreationClassName", new CIMValue(RAIDDiskDriveProvider.CIM_PHYSICAL_CREATION_CLASS_NAME));
      op2.addKey("Tag", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + drive.getAdapter().getDisplayID() + ":ChannelID:" + drive.getChannel().getDisplayID() + ":DeviceID:" + drive.getDeviceID() + ":DiskDrive")));

      instance.setProperty("Dependent", new CIMValue(op2));

      return instance;
   }
}
