package com.ibm.sysmgt.raidmgr.cim.provider.v2.association;

import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import javax.wbem.cim.*;
import javax.wbem.provider.*;

public abstract class AssociatorProviderBase extends RAIDProvider implements CIMAssociatorProvider, CIMInstanceProvider
{
   protected static PrintWriter associationLog = setupLog("AssociatorProvider.log");

   public String CIM_CREATION_CLASS_NAME = "NULL";
   public static final String ANTECEDENT = "Antecedent";
   public static final String DEPENDENT  = "Dependent";

   public AssociatorProviderBase(String logHeader, String creationClassName)
   {
      super(logHeader);
      CIM_CREATION_CLASS_NAME = creationClassName;
   }

   public CIMInstance[] associators(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                             String role, String resultRole, boolean includeQaulifiers, boolean includeClassOrigin,
                             String[] propertyList) throws CIMException
   {
      associationLog.println(logHeader + "associators()");

      Vector associators = new Vector();
      Vector instances = associatorHelper(assocName, objectName, resultClass, role, resultRole);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         associators.addElement(ci.filterProperties(propertyList, includeQaulifiers, includeClassOrigin));
      }

      return (CIMInstance[])associators.toArray(new CIMInstance[associators.size()]);
   }

   public CIMObjectPath[] associatorNames(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException
   {
      associationLog.println(logHeader + "associatorNames()");

      Vector names = new Vector();
      Vector instances = associatorHelper(assocName, objectName, resultClass, role, resultRole);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         // Create a vector of Names from the instances returned from the helper method
         CIMInstance instance = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(instance.getClassName(), objectName.getNameSpace());
         cop.setKeys(instance.getKeyValuePairs());
         names.addElement(cop);
      }

      return (CIMObjectPath[])names.toArray(new CIMObjectPath[names.size()]);
   }
   
   protected abstract Vector associatorHelper(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException;

   public CIMInstance[] references(CIMObjectPath assocName, CIMObjectPath objectName,
                            String role, boolean includeQaulifiers, boolean includeClassOrigin,
                            String[] propertyList) throws CIMException
   {
      associationLog.println(logHeader + "references()");

      Vector references = new Vector();
      Vector instances = referenceHelper(assocName, objectName, role);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         references.addElement(ci.filterProperties(propertyList, includeQaulifiers, includeClassOrigin));
      }

      return (CIMInstance[])references.toArray(new CIMInstance[references.size()]);
   }

   public CIMObjectPath[] referenceNames(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException
   {
      associationLog.println(logHeader + "referenceNames()");

      Vector names = new Vector();
      Vector instances = referenceHelper(assocName, objectName, role);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         // Create a vector of Names from the instances returned from the helper method
         CIMInstance instance = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(instance.getClassName(), objectName.getNameSpace());
         cop.setKeys(instance.getKeyValuePairs());
         names.addElement(cop);
      }

      return (CIMObjectPath[])names.toArray(new CIMObjectPath[names.size()]);
   }

   protected abstract Vector referenceHelper(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException;

   //*************************************************************************************************************
   // Instance Provider Implementation
   //    Windows does not support the AssociatorProvider interface but instead wants to enumerate instances of
   //    the association classes.
   //*************************************************************************************************************
   
   /** Satisfies the InstanceProvider interface */
   public CIMObjectPath createInstance(CIMObjectPath op, CIMInstance ci) throws CIMException
   {
      associationLog.println(logHeader + "createInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the InstanceProvider interface */
   public void deleteInstance(CIMObjectPath op) throws CIMException
   {
      associationLog.println(logHeader + "deleteInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the CIMInstanceProvider interface
     * Enumerates only the names of instances of the class specified by op
     */
   public abstract CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException;

   /** Satisfies the CIMInstanceProvider interface
     * Enumerates the instances of the class specified by op
     */
   public abstract CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException;
   
   /** Satisfies the InstanceProvider interface */
   public CIMInstance[] execQuery(CIMObjectPath op, String query, String ql, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "execQuery()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the InstanceProvider interface */
   public CIMInstance getInstance(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      CIMObjectPath antecedent = (CIMObjectPath)ProviderUtil.getKeyValue(op, "Antecedent");
      CIMObjectPath dependent = (CIMObjectPath)ProviderUtil.getKeyValue(op, "Dependent");

      CIMInstance instance = cc.newInstance();

      instance.setProperty("Antecedent", new CIMValue((CIMObjectPath) ProviderUtil.getKeyValue(op, "Antecedent")));
      instance.setProperty("Dependent", new CIMValue((CIMObjectPath) ProviderUtil.getKeyValue(op, "Dependent")));

      return instance;
   }
   
   /** Satisfies the InstanceProvider interface */
   public void setInstance(CIMObjectPath op, CIMInstance ci, boolean includeQualifiers, String[] propertyList) throws CIMException
   {
      associationLog.println(logHeader + "setInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

}


