
package com.ibm.sysmgt.raidmgr.cim.provider.v2.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;
import com.ibm.sysmgt.storage.api.StorRet;

import javax.wbem.cim.*;
import javax.wbem.provider.*;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class AdaptecRaidAdapterProvider extends InstanceProviderBase implements CIMMethodProvider
{
   public static final String CIM_CREATION_CLASS_NAME             = "ADPT_AdaptecController";
   public static final String CIM_CARD_CREATION_CLASS_NAME        = "ADPT_AdaptecControllerCard";
   public static final String CIM_BIOS_CREATION_CLASS_NAME        = "ADPT_AdaptecBIOS";
   public static final String CIM_FIRMWARE_CREATION_CLASS_NAME    = "ADPT_AdaptecFirmware";
   public static final String CIM_DRIVER_CREATION_CLASS_NAME      = "ADPT_AdaptecDriver";

   public AdaptecRaidAdapterProvider()
   {
      super("AdaptecAdapterInstanceProvider:");
      instanceLog.println(logHeader + "AdaptecRaidAdapterProvider()");
   }

   public CIMValue invokeMethod (CIMObjectPath op, String methodName, CIMArgument[] inArgs, CIMArgument[] outArgs) throws CIMException
   {
      instanceLog.println(logHeader + "invokeMethod(" + methodName + ")");

      if (!methodName.equalsIgnoreCase("RestoreProperties"))
      {
         instanceLog.println(logHeader + "Does not provide invokeMethod for method " + methodName);
         return new CIMValue(new UnsignedInt32(1 /* not supported */));
      }

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         instanceLog.println(logHeader + "Does not provide invokeMethod for objects of type " + op.getObjectName());
         return new CIMValue(new UnsignedInt32(1 /* not supported */));
      }

      int controllerID = 0;

      String name = (String)ProviderUtil.getKeyValue(op, "Name");
      controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

      try
      {
         Object[] args = {new Integer(controllerID), new Boolean(true)};      
         StorRet ret = (StorRet)getDataProc().invokeMethod("resetAdapterRaidConfig", args);      
         return new CIMValue(new UnsignedInt32(ret.iReturnCode));
      }
      catch (RemoteException ex)
      {  
         instanceLog.println(ex);
         ex.printStackTrace(instanceLog);
         return new CIMValue(new UnsignedInt32(2 /* failure */));
      }
   }

   public CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstancesNames(" + op.getObjectName() + ")");

      if (true)
      {
         return new CIMObjectPath[] {};
      }

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
         instances = enumRAIDAdapters(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_CARD_CREATION_CLASS_NAME))
         instances = enumRAIDCards(system, cc);
      /*else if (op.getObjectName().equalsIgnoreCase(CIM_BIOS_CREATION_CLASS_NAME))
         instances = enumRAIDAdapterBIOS(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_FIRMWARE_CREATION_CLASS_NAME))
         instances = enumRAIDAdapterFirmware(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_DRIVER_CREATION_CLASS_NAME))
         instances = enumRAIDAdapterDriver(system, cc);*/
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());        
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }
      
      return (CIMObjectPath[])cops.toArray(new CIMObjectPath[cops.size()]);
   }

   public CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");      

      if (true)
      {
         return new CIMInstance[] {};
      }

      RaidSystem system = null;
      try
      {         
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {         
         Vector v = enumRAIDAdapters(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_CARD_CREATION_CLASS_NAME))
      {         
         Vector v = enumRAIDCards(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      /*else if (op.getObjectName().equalsIgnoreCase(CIM_BIOS_CREATION_CLASS_NAME))
      {               
         Vector v =  enumRAIDAdapterBIOS(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_FIRMWARE_CREATION_CLASS_NAME))
      {               
         Vector v = enumRAIDAdapterFirmware(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_DRIVER_CREATION_CLASS_NAME))
      {               
         Vector v = enumRAIDAdapterDriver(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }*/
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");      

      if (true)
      {
         return null;
      }

      RaidSystem system = null;
      try
      {
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
         return getRAIDAdapter(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_CARD_CREATION_CLASS_NAME))
         return getRAIDCard(system, cc, op);
      /*else if (op.getObjectName().equalsIgnoreCase(CIM_BIOS_CREATION_CLASS_NAME))         
         return getRAIDAdapterBIOS(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_FIRMWARE_CREATION_CLASS_NAME))
         return getRAIDAdapterFirmware(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_DRIVER_CREATION_CLASS_NAME))
         return getRAIDAdapterDriver(system, cc, op);
      */
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // RAID Adapter
   // **********************
   public Vector enumRAIDAdapters(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         if (adapter instanceof ServeRaidAdapter)
            instances.addElement(createCIMInstance((ServeRaidAdapter)adapter, cc.newInstance()));
      }

      return instances;
   }

   public CIMInstance getRAIDAdapter(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String name = (String)ProviderUtil.getKeyValue(op, "Name");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

         ServeRaidAdapter adapter = (ServeRaidAdapter)system.getAdapter(controllerID);
         return createCIMInstance(adapter, cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // RAID Card
   // **********************
   public Vector enumRAIDCards(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         if (adapter instanceof ServeRaidAdapter)
            instances.addElement(createCardInstance((ServeRaidAdapter)adapter, cc.newInstance()));
      }

      return instances;
   }

   public CIMInstance getRAIDCard(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String name = (String)ProviderUtil.getKeyValue(op, "Tag");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

         ServeRaidAdapter adapter = (ServeRaidAdapter)system.getAdapter(controllerID);
         return createCardInstance(adapter, cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }


   // **********************
   // RAID Adapter BIOS
   // **********************
   /*public Vector enumRAIDAdapterBIOS(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         if (adapter instanceof ServeRaidAdapter)
            instances.addElement(((ServeRaidAdapter)adapter).createCIMBIOSInstance(cc.newInstance()));
      }

      return instances;
   }

   public CIMInstance getRAIDAdapterBIOS(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {
         int controllerID = Adapter.displayIDToAdapterID((String)ProviderUtil.getKeyValue(op, "SoftwareElementID"));         
         return ((ServeRaidAdapter)system.getAdapter(controllerID)).createCIMBIOSInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }
   */
   // **********************
   // RAID Adapter Firmware
   // **********************
   /*public Vector enumRAIDAdapterFirmware(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         if (adapter instanceof ServeRaidAdapter)
            instances.addElement(((ServeRaidAdapter)adapter).createCIMFirmwareInstance(cc.newInstance()));
      }

      return instances;
   }

   public CIMInstance getRAIDAdapterFirmware(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {
         int controllerID = Adapter.displayIDToAdapterID((String)ProviderUtil.getKeyValue(op, "SoftwareElementID"));
         return ((ServeRaidAdapter) system.getAdapter(controllerID)).createCIMFirmwareInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }
   */
   // **********************
   // RAID Adapter Driver
   // **********************
   /*public Vector enumRAIDAdapterDriver(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         if (adapter instanceof ServeRaidAdapter)
            instances.addElement(((ServeRaidAdapter)adapter).createCIMDriverInstance(cc.newInstance()));
      }

      return instances;
   }

   public CIMInstance getRAIDAdapterDriver(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {
         int controllerID = Adapter.displayIDToAdapterID((String)ProviderUtil.getKeyValue(op, "SoftwareElementID"));
         return ((ServeRaidAdapter) system.getAdapter(controllerID)).createCIMDriverInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }
    */

    public CIMInstance createCIMInstance(ServeRaidAdapter adapter, CIMInstance instance) throws CIMException
    {       
       // Keys
       instance.setProperty("Name", new CIMValue("ADPT:ServeRAID:ControllerID:" + adapter.getDisplayID()));
       instance.setProperty("CreationClassName", new CIMValue(CIM_CREATION_CLASS_NAME));

       instance.setProperty("ADPTControllerID", new CIMValue(adapter.getDisplayID()));
       instance.setProperty("Caption", new CIMValue("IBM ServeRAID Controller"));
       instance.setProperty("Description", new CIMValue(JCRMUtil.getNLSString(adapter.getAdapterTypeKey())));
       instance.setProperty("NameFormat", new CIMValue("Other"));

       Vector v = new Vector();
       // SMI Spec says to use "Array" but this doesn't seem to exist.
       v.addElement(new UnsignedInt16(3) /* Storage */);
       v.addElement(new UnsignedInt16(15) /* Block Server */);
       instance.setProperty("Dedicated", new CIMValue(v, ProviderUtil.UINT16_ARRAY_TYPE));

       if (adapter.getStatus() == Constants.WORKING)
          instance.setProperty("Status", new CIMValue(JCRMUtil.getNLSString("cimOK")));
       else
          instance.setProperty("Status", new CIMValue(JCRMUtil.getNLSString("cimNonRecover")));
      
       // Working adapter, overall status is OK
       if (adapter.getStatus() == Constants.WORKING && adapter.getOverallStatus() == Constants.NORMAL)
       {
          Vector v1 = new Vector();
          v1.addElement(new UnsignedInt16(2 /* OK */));
          instance.setProperty("OperationalStatus", new CIMValue(v1, ProviderUtil.UINT16_ARRAY_TYPE));
       }
       // Bad adapter
       else if (adapter.getStatus() != Constants.WORKING)       
       {
          Vector v1 = new Vector();
          v1.addElement(new UnsignedInt16(7 /* Non-Recoverable Error */));
          instance.setProperty("OperationalStatus", new CIMValue(v1, ProviderUtil.UINT16_ARRAY_TYPE));
       }
       // Working adapter, bad overall status
       else if (adapter.getStatus() == Constants.WORKING && adapter.getOverallStatus() != Constants.NORMAL)
       {
          Vector v1 = new Vector();
          v1.addElement(new UnsignedInt16(2) /* OK */);
          v1.addElement(new UnsignedInt16(16) /* Supporting Entity in Error */);
          instance.setProperty("OperationalStatus", new CIMValue(v1, ProviderUtil.UINT16_ARRAY_TYPE));
       }          

       instance.setProperty("ResetCapability", new CIMValue(new UnsignedInt16(5 /* Not Implemented */)));

       Vector v1 = new Vector();
       v1.addElement(new UnsignedInt16(4) /* Power Saving Modes Entered Automatically */);
       instance.setProperty("PowerManagementCapabilities", new CIMValue(v1, ProviderUtil.UINT16_ARRAY_TYPE));       

       return instance;
    }

    public CIMInstance createCardInstance(ServeRaidAdapter adapter, CIMInstance instance) throws CIMException
    {       
       // Keys
       instance.setProperty("Tag", new CIMValue("ADPT:ServeRAID:ControllerID:" + adapter.getDisplayID()));
       instance.setProperty("CreationClassName", new CIMValue(CIM_CARD_CREATION_CLASS_NAME));

       instance.setProperty("Caption", new CIMValue("IBM ServeRAID Controller"));
       instance.setProperty("Description", new CIMValue(JCRMUtil.getNLSString(adapter.getAdapterTypeKey())));
       instance.setProperty("ADPTControllerID", new CIMValue(adapter.getDisplayID()));
       instance.setProperty("Manufacturer", new CIMValue("IBM"));
       instance.setProperty("Model", new CIMValue(JCRMUtil.getNLSString(adapter.getAdapterTypeKey())));
       instance.setProperty("SerialNumber", new CIMValue(adapter.getVPDInfo().getSerialNumber()));
       instance.setProperty("PartNumber", new CIMValue(adapter.getVPDInfo().getPartNumber()));
       instance.setProperty("PoweredOn", new CIMValue(new Boolean(true)));
       instance.setProperty("HostingBoard", new CIMValue(new Boolean(false)));
       instance.setProperty("RequiresDaughterBoard", new CIMValue(new Boolean(false)));

       if (adapter.getAdapterType() == Constants.NAVAJO_1 || adapter.getAdapterType() == Constants.KIOWA_1)
       {
          instance.setProperty("Removable", new CIMValue(new Boolean(false)));
          instance.setProperty("Replaceable", new CIMValue(new Boolean(false)));
          instance.setProperty("HotSwappable", new CIMValue(new Boolean(false)));
       }          
       else
       {
          instance.setProperty("Removable", new CIMValue(new Boolean(true)));
          instance.setProperty("Replaceable", new CIMValue(new Boolean(true)));
          instance.setProperty("HotSwappable", new CIMValue(new Boolean(true)));
       }          

       instance.setProperty("BusType", new CIMValue(new UnsignedInt16(43) /* PCI */));

       return instance;
    }
}
