
package com.ibm.sysmgt.raidmgr.cim.provider.v2.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;
import com.ibm.sysmgt.storage.api.StorRet;

import javax.wbem.cim.*;
import javax.wbem.provider.*;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;
import java.math.BigInteger;

public class ArrayProfileProvider extends InstanceProviderBase implements CIMMethodProvider
{
   public static final String CIM_STORAGE_CONFIGURATION_SERVICE = "ADPT_StorageConfigurationService";
   public static final String CIM_STORAGE_POOL                  = "ADPT_StoragePool";
   public static final String CIM_STORAGE_CAPABILITIES          = "ADPT_StorageCapabilities";
   public static final String CIM_STORAGE_VOLUME                = "ADPT_StorageVolume";
   public static final String CIM_STORAGE_SETTING               = "ADPT_StorageSetting";

   public ArrayProfileProvider()
   {
      super("ArrayProfileProvider:");
      instanceLog.println(logHeader + "ArrayProfileProvider()");
   }

   public CIMValue invokeMethod (CIMObjectPath op, String methodName, CIMArgument[] inArgs, CIMArgument[] outArgs) throws CIMException
   {
      instanceLog.println(logHeader + "invokeMethod(" + methodName + ")");
      
      if (methodName.equalsIgnoreCase("GetSupportedSizeRange"))
         return getSupportedSizeRange(op, methodName, inArgs, outArgs);
      else if (methodName.equalsIgnoreCase("CreateOrModifyElementFromStoragePool"))
         return createOrModifyElementFromStoragePool(op, methodName, inArgs, outArgs);
      else if (methodName.equalsIgnoreCase("GetSupportedSizeRange"))
         return returnToStoragePool(op, methodName, inArgs, outArgs);
      else
      {
         instanceLog.println(logHeader + "Does not provide invokeMethod for method " + methodName);
         return new CIMValue(new UnsignedInt32(1 /* not supported */));
      }
   }

   public CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstancesNames(" + op.getObjectName() + ")");

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();         
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CONFIGURATION_SERVICE))
          instances = enumStorageConfigurationServices(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_POOL))
         instances = enumStoragePools(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CAPABILITIES))
         instances = enumStorageCapabilities(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_VOLUME))
         instances = enumStorageVolumes(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_SETTING))
         instances = enumStorageSettings(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());        
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }
      
      return (CIMObjectPath[])cops.toArray(new CIMObjectPath[cops.size()]);
   }

   public CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");      

      RaidSystem system = null;
      try
      {         
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CONFIGURATION_SERVICE))
      {
         Vector v = enumStorageConfigurationServices(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_POOL))
      {         
         Vector v = enumStoragePools(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CAPABILITIES))
      {         
         Vector v = enumStorageCapabilities(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_VOLUME))
      {               
         Vector v =  enumStorageVolumes(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_SETTING))
      {               
         Vector v = enumStorageSettings(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");      

      RaidSystem system = null;
      try
      {
         system = getRaidSystem();         
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CONFIGURATION_SERVICE))
         return getStorageConfigurationService(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_POOL))
         return getStoragePool(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CAPABILITIES))
         return getStorageCapability(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_VOLUME))
         return getStorageVolume(system, cc, op);
      else if (op.getObjectName().equalsIgnoreCase(CIM_STORAGE_SETTING))
         return getStorageSetting(system, cc, op);
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // *****************************
   // StorageConfigurationService
   // *****************************
   public Vector enumStorageConfigurationServices(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
         instances.addElement(createStorageConfigurationService((Adapter)e.nextElement(), cc.newInstance()));

      return instances;
   }

   public CIMInstance getStorageConfigurationService(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String instanceID = (String)ProviderUtil.getKeyValue(op, "Name");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(instanceID, ":", "ControllerID"));
         
         return createStorageConfigurationService(system.getAdapter(controllerID), cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }   
   
   // **********************
   // StoragePool
   // **********************
   public Vector enumStoragePools(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
         instances.addElement(createStoragePoolInstance((Adapter)e.nextElement(), cc.newInstance()));

      return instances;
   }

   public CIMInstance getStoragePool(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String instanceID = (String)ProviderUtil.getKeyValue(op, "InstanceID");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(instanceID, ":", "ControllerID"));                 

         return createStoragePoolInstance(system.getAdapter(controllerID), cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // StorageCapabilities
   // **********************
   public Vector enumStorageCapabilities(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
         instances.addElement(createStorageCapabilitiesInstance((Adapter)e.nextElement(), cc.newInstance()));

      return instances;
   }

   public CIMInstance getStorageCapability(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String name = (String)ProviderUtil.getKeyValue(op, "Tag");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

         return createStorageCapabilitiesInstance(system.getAdapter(controllerID), cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // StorageVolume
   // **********************
   public Vector enumStorageVolumes(RaidSystem system, CIMClass cc) throws CIMException
   {
      instanceLog.println("(ArrayProfileProvider) inside enumStorageVolumes()");

      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();         
         instanceLog.println("(enumStorageVolumes) adapter.getLogicalDriveCount() = " + adapter.getLogicalDriveCount());
         instanceLog.println("(enumStorageVolumes) adapter.enumerateLogicalDrives().hasMoreElements() = " + adapter.enumerateLogicalDrives().hasMoreElements());

         for (Enumeration f = adapter.enumerateLogicalDrives(); f.hasMoreElements();)
         {  
            LogicalDrive ld = (LogicalDrive)f.nextElement();            
            if (ld.getState() != Constants.FRE)
               instances.addElement(createStorageVolumeInstance(adapter, ld, cc.newInstance()));
         }
      }         

      return instances;
   }

   public CIMInstance getStorageVolume(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {            
      String name = (String)ProviderUtil.getKeyValue(op, "DeviceID");
      int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));
      int ldID = LogicalDrive.displayIDToLogicalDriveID(ProviderUtil.getToken(name, ":", "StorageVolumeID"));

      try
      {                  
         Adapter adapter = system.getAdapter(controllerID);         

         Vector v = adapter.getLogicalDriveCollection(null);

         // Since we do a getConfigAll we have both HLDs and SLDs in the Vector v
         // We need to find the correct logical drive since there are two elements in the
         // vector with the same logical drive ID, one SLD and one HLD.
         for (Enumeration e = v.elements(); e.hasMoreElements();)
         {
            LogicalDrive ld = (LogicalDrive) e.nextElement();
            if (ld.getID() == ldID && ld.getState() != Constants.FRE)
               return createStorageVolumeInstance(adapter, ld, cc.newInstance());
         }                                    
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }

      instanceLog.println("!!! getStorageVolume failed to locate LogicalDrive " + ldID);
      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
   }

   // **********************
   // StorageSetting
   // **********************
   public Vector enumStorageSettings(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      //for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      //   instances.addElement(createStorageSettingInstance((Adapter)e.nextElement(), cc.newInstance()));

      return instances;
   }

   public CIMInstance getStorageSetting(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {      
      try
      {                  
         String name = (String)ProviderUtil.getKeyValue(op, "Tag");
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

         return createStorageSettingInstance(system.getAdapter(controllerID), cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }


   public CIMInstance createStorageConfigurationService(Adapter adapter, CIMInstance instance) throws CIMException
   {
      instance.setProperty("SystemName", new CIMValue(JCRMNet.getHostname()));
      instance.setProperty("SystemCreationClassName", new CIMValue(ProviderUtil.getSystemCreationClassName()));
      instance.setProperty("CreationClassName", new CIMValue(CIM_STORAGE_CONFIGURATION_SERVICE));
      instance.setProperty("Name", new CIMValue("ADPT:ControllerID:" + adapter.getDisplayID()));
     
      instance.setProperty("Caption", new CIMValue("Adaptec RAID Configuration Service"));
      instance.setProperty("Description", new CIMValue("Adaptec RAID Configuration Service"));

      Vector v = new Vector();
      v.addElement(new UnsignedInt16(1 /* OK */));
      instance.setProperty("OperationalStatus", new CIMValue(v, ProviderUtil.UINT16_ARRAY_TYPE));
      instance.setProperty("Started", new CIMValue(new Boolean(true)));

      return instance;
   }


    public CIMInstance createStoragePoolInstance(Adapter adapter, CIMInstance instance) throws CIMException
    {       
       // Keys
       instance.setProperty("InstanceID", new CIMValue("ADPT:ControllerID:" + adapter.getDisplayID()));       

       instance.setProperty("Caption", new CIMValue("Adaptec Controller " + adapter.getDisplayID() + " Storage Pool"));       
       instance.setProperty("Description", new CIMValue("Adaptec Controller " + adapter.getDisplayID() + " Storage Pool"));       
       instance.setProperty("PoolID", new CIMValue("Pool:" + adapter.getDisplayID()));       

       try
       {
          BigInteger freeSpace = BigInteger.ZERO;          
          for (Enumeration e = adapter.getPhysicalDeviceCollection(null).elements(); e.hasMoreElements();)
          {
             try
             {                
                HardDrive hd = (HardDrive)e.nextElement();
                if (hd.getState() == Constants.RDY)
                   freeSpace = freeSpace.add(new BigInteger(Integer.toString(hd.getSize())));
             }
             catch (ClassCastException ex)
             {
                // Not a HardDrive device, continue
             }
          }
          
          instance.setProperty("TotalAvailableSpace", new CIMValue(new UnsignedInt64(freeSpace)));
       }
       catch (NumberFormatException ex)
       {          
          ex.printStackTrace(instanceLog);
       }       

       return instance;
    }

    public CIMInstance createStorageCapabilitiesInstance(Adapter adapter, CIMInstance instance) throws CIMException
    {       
       // Keys
       instance.setProperty("Name", new CIMValue("ADPT:ServeRAID:ControllerID:" + adapter.getDisplayID()));
       instance.setProperty("CreationClassName", new CIMValue(CIM_STORAGE_CAPABILITIES));

       return instance;
    }

    public CIMInstance createStorageVolumeInstance(Adapter adapter, LogicalDrive logicalDrive, CIMInstance instance) throws CIMException
    {       
       // Keys       
       instance.setProperty("CreationClassName", new CIMValue(CIM_STORAGE_VOLUME));
       instance.setProperty("DeviceID", new CIMValue("ADPT:ControllerID:" + adapter.getDisplayID() + ":StorageVolumeID:" + logicalDrive.getDisplayID()));
       instance.setProperty("SystemName", new CIMValue(JCRMNet.getHostname()));
       instance.setProperty("SystemCreationClassName", new CIMValue(ProviderUtil.getSystemCreationClassName()));

       instance.setProperty("Caption", new CIMValue("Adaptec Storage Volume " + logicalDrive.getDisplayID()));
       instance.setProperty("Description", new CIMValue("Adaptec Storage Volume " + logicalDrive.getDisplayID()));

       Vector v = new Vector();

       int ldState = logicalDrive.getState();
       instanceLog.println("adapter.getID() = " + adapter.getID());
       instanceLog.println("logicalDrive.getID() = " + logicalDrive.getID());
       instanceLog.println("logicalDrive.getState() = " + ldState);

       if (ldState == Constants.OFL)
       {
          v.addElement(new UnsignedInt16(7 /* Non-Recoverable Error */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(8 /* Off LIne */)));
       }
       else if (ldState == Constants.OKY)
       {
          instanceLog.println("OK!!!!!!");
          v.addElement(new UnsignedInt16(2 /* OK */));                              
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(3 /* Running/Full Power */))); 
       }
       else if (ldState == Constants.CRT)
       {
          v.addElement(new UnsignedInt16(3 /* Degraded */));
          v.addElement(new UnsignedInt16(6 /* Error */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(4 /* Warning */)));
       }
       else if (ldState == Constants.LDM)
       {
          v.addElement(new UnsignedInt16(2 /* OK */));
          v.addElement(new UnsignedInt16(11/* In Service */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(3 /* Running/Full Power */)));
       }
       else if (ldState == Constants.SYS)
       {
          v.addElement(new UnsignedInt16(1 /* OK */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(3 /* Running/Full Power */)));
       }
       else if (ldState == Constants.CRM)
       {       
          v.addElement(new UnsignedInt16(3 /* Degraded */));
          v.addElement(new UnsignedInt16(6 /* Error */)); 
          v.addElement(new UnsignedInt16(11/* In Service */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(4 /* Warning */)));
       }
       else if (ldState == Constants.CRS)
       {
          v.addElement(new UnsignedInt16(3 /* Degraded */));
          v.addElement(new UnsignedInt16(6 /* Error */));
          instance.setProperty("Availability", new CIMValue(new UnsignedInt16(4 /* Warning */)));
       }
       else
          v.addElement(new UnsignedInt16(0 /* Unknown */));

       for (Enumeration blah = v.elements(); blah.hasMoreElements();)
       {
         UnsignedInt16 value = (UnsignedInt16)blah.nextElement();
         instanceLog.println("wtf = " + value);
       }

       instance.setProperty("OperationalStatus", new CIMValue(v, ProviderUtil.UINT16_ARRAY_TYPE));
       instance.setProperty("Access", new CIMValue(new UnsignedInt16(3 /* Read/Write */)));
       instance.setProperty("BlockSize", new CIMValue(new UnsignedInt64(new BigInteger("1" /* Unkown/FixMe */))));

       int raidLevel = logicalDrive.getRaidLevel();
       int numSpares = 0;

       instance.setProperty("ADPTRaidLevel", new CIMValue(logicalDrive.getDisplayRaidLevel()));

       // The SMIS states that SpindleRedundancy can include at most one hot spare in the calculation,
       // so if the hot spare enumeration returns 1 element or 100, numSpares will remain 1.
       if (logicalDrive.enumerateHotSpares().hasMoreElements())
          numSpares = 1;

       if (raidLevel == Constants.RAID0 || raidLevel == Constants.RAID00)
       {       
          instance.setProperty("IsBasedOnUnderlyingRedundancy", new CIMValue(new Boolean(false)));
          instance.setProperty("NoSinglePointOfFailure", new CIMValue(new Boolean(false)));
          instance.setProperty("DataRedundancy", new CIMValue(new UnsignedInt16(1))); 
          instance.setProperty("SpindleRedundancy", new CIMValue(new UnsignedInt16(0)));
       }
       else if (raidLevel > 100 /* Hierarchical Raid excluding RAID00 */)
       {
          instance.setProperty("IsBasedOnUnderlyingRedundancy", new CIMValue(new Boolean(true)));
          instance.setProperty("NoSinglePointOfFailure", new CIMValue(new Boolean(true)));
          instance.setProperty("DataRedundancy", new CIMValue(new UnsignedInt16(2)));
          instance.setProperty("SpindleRedundancy", new CIMValue(new UnsignedInt16(2 + numSpares)));
       }
       else
       {       
          instance.setProperty("IsBasedOnUnderlyingRedundancy", new CIMValue(new Boolean(true)));
          instance.setProperty("NoSinglePointOfFailure", new CIMValue(new Boolean(true)));
          instance.setProperty("DataRedundancy", new CIMValue(new UnsignedInt16(2)));
          instance.setProperty("SpindleRedundancy", new CIMValue(new UnsignedInt16(1 + numSpares)));
       }

       instance.setProperty("SequentialAccess", new CIMValue(new Boolean(false)));       

       return instance;
    }

    public CIMInstance createStorageSettingInstance(Adapter adapter, CIMInstance instance) throws CIMException
    {       
       // Keys
       //instance.setProperty("Name", new CIMValue("ADPT:ServeRAID:ControllerID:" + adapter.getDisplayID()));
       //instance.setProperty("CreationClassName", new CIMValue(CIM_STORAGE_SETTING));

       return instance;
    }

    private CIMValue getSupportedSizeRange(CIMObjectPath op, String methodName, CIMArgument[] inArgs, CIMArgument[] outArgs) throws CIMException
    {

       if (!op.getObjectName().equalsIgnoreCase(CIM_STORAGE_POOL))
       {
          instanceLog.println(logHeader + "Does not provide invokeMethod for objects of type " + op.getObjectName());
          return new CIMValue(new UnsignedInt32(1 /* not supported */));
       }

       int controllerID = 0;

       String name = (String)ProviderUtil.getKeyValue(op, "InstanceID");
       controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

       RaidSystem system = null;
       try
       {         
          system = (RaidSystem)getRaidSystem();
       }
       catch (RemoteException e)
       {
          instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
          throw new CIMException(CIMException.CIM_ERR_FAILED);
       }

       try
       {
          CIMInstance inStorageSettingInstance = ProviderUtil.getInstanceFromRefArgument(cimom, inArgs[0]);
       }      
       catch (CIMException ex)
       {
          ex.printStackTrace(instanceLog);
          throw ex;
       }
       catch (IndexOutOfBoundsException ex)
       {
          ex.printStackTrace(instanceLog);
          throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER);
       }

       // FixMe - Temporary Code
       CIMValue minVolumeSize = new CIMValue(new UnsignedInt64(new BigInteger("1")));
       CIMValue maxVolumeSize = new CIMValue(new UnsignedInt64(new BigInteger("1024")));
       CIMValue volumeDivisor = new CIMValue(new UnsignedInt64(new BigInteger("1")));

       outArgs[0] = new CIMArgument("MinimumVolumeSize", minVolumeSize);
       outArgs[1] = new CIMArgument("MaximumVolumeSize" , maxVolumeSize);
       outArgs[2] = new CIMArgument("VolumeSizeDivisor", volumeDivisor);

       return new CIMValue(new UnsignedInt32(0 /* success */));
    }

   private CIMValue createOrModifyElementFromStoragePool(CIMObjectPath op, String methodName, CIMArgument[] inArgs, CIMArgument[] outArgs) throws CIMException
   {
      if (!op.getObjectName().equalsIgnoreCase(CIM_STORAGE_CONFIGURATION_SERVICE))
      {
         instanceLog.println(logHeader + "(createOrModifyElementFromStoragePool) Does not provide invokeMethod for objects of type " + op.getObjectName());
         return new CIMValue(new UnsignedInt32(1 /* not supported */));
      }

      // Input Parameters
      CIMArgument elementTypeIN = inArgs[0];    /* UnsignedInt16 */
      CIMArgument storageSettingIN = inArgs[1]; /* CIM_StorageSetting REF */
      CIMArgument sizeIN = inArgs[2];           /* UnsignedInt64 */
      CIMArgument storagePoolIN = inArgs[3];    /* CIM_StoragePool REF */
      CIMArgument theElementIN = inArgs[4];     /* CIM_LogicalElement REF */

      // Only support StorageVolume
      UnsignedInt16 elementType = (UnsignedInt16)elementTypeIN.getValue().getValue();
      if (elementType.intValue() != 2 /* StorageVolume */)
      {
         instanceLog.println("!!! (createOrModifyElementFromStoragePool) Element Type must be 2 (StorageVolume)");
         return new CIMValue(new UnsignedInt32(1 /* Not Supported */));
      }
         
      CIMInstance inStoragePool = ProviderUtil.getInstanceFromRefArgument(cimom, storagePoolIN);

      int controllerID = 0;


      instanceLog.println("inStorageSetting.getObjectPath() = " + ProviderUtil.getInstanceFromRefArgument(cimom, storageSettingIN).getObjectPath());
      instanceLog.println("inStoragePool.getObjectPath() = " + ProviderUtil.getInstanceFromRefArgument(cimom, storagePoolIN).getObjectPath());      
      String name = (String)ProviderUtil.getKeyValue(ProviderUtil.getInstanceFromRefArgument(cimom, storagePoolIN).getObjectPath(), "InstanceID");
      controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(name, ":", "ControllerID"));

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }
           
      Adapter adapter = system.getAdapter(controllerID);
      if (adapter == null)
      {
         instanceLog.println("!!! (createOrModifyElementFromStoragePool) system.getAdapter(" + controllerID + ") returned null");
         return new CIMValue(new UnsignedInt32(1 /* failure */));
      }

      ArrayProfileProviderHelper.getRaidLevelFromStorageSetting(adapter, storageSettingIN.getType(), ProviderUtil.getInstanceFromRefArgument(cimom, storageSettingIN));

      return new CIMValue(new UnsignedInt32(0 /* success */));
   }

   private CIMValue returnToStoragePool(CIMObjectPath op, String methodName, CIMArgument[] inArgs, CIMArgument[] outArgs) throws CIMException
   {
      return new CIMValue(new UnsignedInt32(1 /* failure */));
      //return new CIMValue(new UnsignedInt32(0 /* success */));
   }
}
