package com.ibm.sysmgt.raidmgr.cim.provider.v2.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;

import javax.wbem.provider.*;
import javax.wbem.cim.*;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

/** This class provides a base instance provider class for the various instance providers.
  * This class takes care of the common instance provider methods.
  */
public abstract class InstanceProviderBase extends RAIDProvider implements CIMInstanceProvider
{
   public static PrintWriter instanceLog = setupLog("InstanceProvider.log");

   public InstanceProviderBase(String logHeader)
   {
      super(logHeader);
   }

   /** Satisfies the CIMInstanceProvider interface */
   public CIMObjectPath createInstance(CIMObjectPath op, CIMInstance ci) throws CIMException
   {
      instanceLog.println(logHeader + "createInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   /** Satisfies the CIMInstanceProvider interface */
   public void deleteInstance(CIMObjectPath op) throws CIMException
   {
      instanceLog.println(logHeader + "deleteInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   /** Satisfies the CIMInstanceProvider interface
     * Enumerates only the names of instances of the class specified by op
     */
   public abstract CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException;

   /** Satisfies the CIMInstanceProvider interface
     * Enumerates the instances of the class specified by op
     */
   public abstract CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException;

   /** Satisfies the CIMInstanceProvider interface */
   public CIMInstance[] execQuery(CIMObjectPath op, String query, String ql, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "execQuery()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   /** Satisfies the CIMInstanceProvider interface */
   public abstract CIMInstance getInstance(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException;

   /** Satisfies the CIMInstanceProvider interface */
   public void setInstance(CIMObjectPath op, CIMInstance ci, boolean includeQualifiers, String[] propertyList) throws CIMException
   {
      instanceLog.println(logHeader + "setInstance()");
      for (int i = 0; i < propertyList.length; i++)
         instanceLog.println("propertyList[" + i + "] = " + propertyList[i]);

      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
}
