
package com.ibm.sysmgt.raidmgr.cim.provider.v2.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;

import javax.wbem.cim.*;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class RAIDDiskDriveProvider extends InstanceProviderBase
{
   public static final String CIM_CREATION_CLASS_NAME             = "ADPT_RAIDDiskDrive";
   public static final String CIM_PHYSICAL_CREATION_CLASS_NAME    = "ADPT_PhysicalRAIDDiskDrive";

   public RAIDDiskDriveProvider()
   {
      super("RAIDDiskDriveProvider:");
      instanceLog.println(logHeader + "RAIDDiskDriveProvider()");
   }

   public CIMObjectPath[] enumerateInstanceNames(CIMObjectPath op, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstanceNames(" + op.getObjectName() + ")");

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
         instances = enumDiskDrives(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_PHYSICAL_CREATION_CLASS_NAME))
         instances = enumPhysicalDiskDrives(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }
      
      return (CIMObjectPath[])cops.toArray(new CIMObjectPath[cops.size()]);
   }

   public CIMInstance[] enumerateInstances(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");      

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {         
         Vector v = enumDiskDrives(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }         
      if (op.getObjectName().equalsIgnoreCase(CIM_PHYSICAL_CREATION_CLASS_NAME))
      {      
         Vector v = enumPhysicalDiskDrives(system, cc);
         return (CIMInstance[])v.toArray(new CIMInstance[v.size()]);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, boolean localOnly, boolean includeQualifiers, boolean includeClassOrigin, String[] propertyList, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");      

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);         
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
         return getDiskDrive(system, cc, op);
      if (op.getObjectName().equalsIgnoreCase(CIM_PHYSICAL_CREATION_CLASS_NAME))
         return getPhysicalDiskDrive(system, cc, op);
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // Disk Drives (logical)
   // **********************
   public Vector enumDiskDrives(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(createCIMInstance(drive, cc.newInstance()));
         }         
      }

      return instances;
   }   

   public CIMInstance getDiskDrive(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)ProviderUtil.getKeyValue(op, "DeviceID");
         int diskDriveID = Integer.parseInt(ProviderUtil.getToken(deviceID, ":", "DiskDriveID"));
         int controllerID = Adapter.displayIDToAdapterID(ProviderUtil.getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(deviceID, ":", "ChannelID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
               return createCIMInstance(drive, cc.newInstance());
         }
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }

      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
   }

   // **********************
   // Disk Drives (physical)
   // **********************
   public Vector enumPhysicalDiskDrives(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(createCIMPhysicalDiskDriveInstance(drive, cc.newInstance()));
         }         
      }

      return instances;
   }   

   public CIMInstance getPhysicalDiskDrive(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String tag = (String)ProviderUtil.getKeyValue(op, "Tag");         

         int controller = Adapter.displayIDToAdapterID(ProviderUtil.getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(ProviderUtil.getToken(tag, ":", "ChannelID"));
         int deviceID = Integer.parseInt(ProviderUtil.getToken(tag, ":", "DeviceID"));

         Adapter adapter = (Adapter)system.getAdapter(controller);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == deviceID)
               return createCIMPhysicalDiskDriveInstance(drive, cc.newInstance());
         }
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }

      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
   }

   public static CIMInstance createCIMInstance(HardDrive drive, CIMInstance instance)
   {
      // Keys
      instance.setProperty("CreationClassName", new CIMValue(CIM_CREATION_CLASS_NAME));
      instance.setProperty("SystemCreationClassName", new CIMValue(ProviderUtil.getSystemCreationClassName()));
      instance.setProperty("SystemName", new CIMValue(JCRMNet.getHostname()));
      instance.setProperty("DeviceID", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + drive.getAdapter().getDisplayID() + ":ChannelID:" + drive.getChannel().getDisplayID() + ":DiskDriveID:" + drive.getDeviceID() + ":DiskDrive")));

      instance.setProperty("ControllerID", new CIMValue(new String(drive.getAdapter().getDisplayID())));
      instance.setProperty("ChannelID", new CIMValue(new String(drive.getChannel().getDisplayID())));
      instance.setProperty("DiskDriveID", new CIMValue(new String(Integer.toString(drive.getDeviceID()))));

      Vector capabilities = new Vector();
      capabilities.addElement(new UnsignedInt16(3)  /* Random Access */);
      capabilities.addElement(new UnsignedInt16(4)  /* Supports Writing */);
      capabilities.addElement(new UnsignedInt16(10) /* SMART Notification */);            
      instance.setProperty("Capabilities", new CIMValue(capabilities, CIMDataType.getPredefinedType(CIMDataType.UINT16_ARRAY)));      

      instance.setProperty("DefaultBlockSize", new CIMValue(new UnsignedInt64(Integer.toString(512))));
      instance.setProperty("MaxBlockSize", new CIMValue(new UnsignedInt64(Integer.toString(512))));
      instance.setProperty("MinBlockSize", new CIMValue(new UnsignedInt64(Integer.toString(512))));
      instance.setProperty("Firmware", new CIMValue(drive.getFirmwareLevel()));

      String status = JCRMUtil.getNLSString("cimUnknown");
      int availability = 2; /* Unknown */
      switch (drive.getState())
      {
         case (Constants.ONL):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 3; /* Running/Full Power */         
            break;
         case (Constants.RDY):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 3; /* Running/Full Power */
            break;
         case (Constants.SBY):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 15; /* Power Save - Standby */
            break;
         case (Constants.HSP):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 3; /* Running/Full Power */
            break;
         case (Constants.SHS):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 15; /* Power Save - Standby */
            break;
         case (Constants.RBL):
            status = JCRMUtil.getNLSString("cimOK");
            availability = 1; /* Other */
            break;
         case (Constants.DDD):
            status = JCRMUtil.getNLSString("cimNonRecover");
            availability = 8; /* Off Line */
            break;
         case (Constants.DHS):
            status = JCRMUtil.getNLSString("cimNonRecover");
            availability = 8; /* Off Line */
            break;
         default:
      };

      if (drive.hasPFAError())
         status = JCRMUtil.getNLSString("cimPFA");

      instance.setProperty("Status", new CIMValue(status));
      instance.setProperty("Availability", new CIMValue(new UnsignedInt16(availability)));
      instance.setProperty("State", new CIMValue(new UnsignedInt16(drive.getState())));

      switch (drive.getWriteCacheEnableStatus())
      {
         case Constants.WCE_ENABLED:
            instance.setProperty("CacheWritePolicy", new CIMValue(new UnsignedInt16(4) /* Write Back */));
            break;
         case Constants.WCE_DISABLED:
            instance.setProperty("CacheWritePolicy", new CIMValue(new UnsignedInt16(3) /* Write Through */));
            break;
         case Constants.WCE_NOTAVAILABLE:
            instance.setProperty("CacheWritePolicy", new CIMValue(new UnsignedInt16(2) /* Not Available */));
            break;
         case Constants.WCE_NOTSUPPORTED:
            instance.setProperty("CacheWritePolicy", new CIMValue(new UnsignedInt16(1) /* Not Supported */));
            break;
         default:
            break;
      }

      instance.setProperty("Warranted", new CIMValue(new Boolean(drive.getVendorSupported())));

      return instance;
   }

   public static CIMInstance createCIMPhysicalDiskDriveInstance(HardDrive drive, CIMInstance instance)
   {
      // Keys
      instance.setProperty("CreationClassName", new CIMValue(CIM_PHYSICAL_CREATION_CLASS_NAME));
      instance.setProperty("Tag", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + drive.getAdapter().getDisplayID() + ":ChannelID:" + drive.getChannel().getDisplayID() + ":DeviceID:" + drive.getDeviceID() + ":DiskDrive")));

      instance.setProperty("Manufacturer", new CIMValue(drive.getVendor()));
      instance.setProperty("Model", new CIMValue(drive.getModel()));
      instance.setProperty("SerialNumber", new CIMValue(drive.getSerialNumber()));
      instance.setProperty("FRU", new CIMValue(drive.getFruNumber()));

      // Convert drive size to bytes
      long capacity = (long)drive.getSize() * 1024 * 1024;

      instance.setProperty("Capacity", new CIMValue(new UnsignedInt64(Long.toString(capacity))));
      instance.setProperty("Removable", new CIMValue(new Boolean(true)));
      instance.setProperty("HotSwappable", new CIMValue(new Boolean(true)));     
      instance.setProperty("MediaType", new CIMValue(new UnsignedInt16(39) /* MediaType - Hard Disk */));

      return instance;
   }

   public static CIMObjectPath getCIMObjectPath(HardDrive drive)
   {
      CIMObjectPath op = new CIMObjectPath(CIM_CREATION_CLASS_NAME);

      op.addKey("CreationClassName", new CIMValue(CIM_CREATION_CLASS_NAME));
      op.addKey("SystemCreationClassName", new CIMValue(ProviderUtil.getSystemCreationClassName()));
      op.addKey("SystemName", new CIMValue(JCRMNet.getHostname()));
      op.addKey("DeviceID", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + drive.getAdapter().getDisplayID() + ":ChannelID:" + drive.getChannel().getDisplayID() + ":DiskDriveID:" + drive.getDeviceID() + ":DiskDrive")));

      return op;
   }

    public static void main(String [] args)
    {
        RAIDDiskDriveProvider provider = new RAIDDiskDriveProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
