package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocArrays.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.Vector;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/** This class is the container object at the Adapter level for Array objects.  The only allowable 
  * children of Arrays are Array objects, as enforced by the add() method in this class.  As a well-
  * behaved RaidObject, this class uses private final fields to enforce identity integrity.
  */
public class IrocArrays extends Arrays
{
   static final long serialVersionUID = -8058178802964278056L;

   /** The IrocArrays constructor.
     * @param adapterIN  The Adapter container of this object
     */
   public IrocArrays(Adapter adapterIN)
   {
      super(adapterIN);
   }
   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      if (getCachedActions() == null)
      {
         Vector actions = new Vector();
         
         actions.addElement(new HelpNowAction(this));
         actions.addElement(new NullAction());
         actions.addElement(new ConfigAdapterAction(getAdapter()));
         actions.addElement(new DeleteArraysAction(getAdapter()));
         actions.addElement(new NullAction());
         actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_ALL_ARRAYS));
         actions.addElement(new PropertiesAction(this));

         setActions(actions);
      }

      return getCachedActions();
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCArrays";
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      IrocArrays arrays = (IrocArrays) object;

      // Everything matched, so we're equal!
      return true;
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }
}
