package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocSpannedArray.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/16/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.sun.wbem.cim.*;

public class IrocSpannedArray extends SpannedArray implements IrocRaidObjectIntf
{
   static final long serialVersionUID = 2051727303598648373L;

   private long uniqueID;
   private String ldPath;

   public IrocSpannedArray(Adapter adapterIN, 
                           int arrayIDIN, 
                           int sizeIN,
                           int freeSpaceIN,
                           long uniqueIDIN, 
                           int stripeSizeIN,
                           String ldPathIN)
   {
      super(adapterIN, arrayIDIN);
      setDynamicSizingEnabled(false);
      setSize(sizeIN);
      setFreeSpace(freeSpaceIN);
      setStripeSize(stripeSizeIN);

      uniqueID = uniqueIDIN;
      ldPath = ldPathIN;
   }

   /** Returns the uniqueID assigned to this array
     * @return the uniqueID assigned to this array
     */
   public long getUniqueID()
   {
      return uniqueID;
   }

   /** Returns the ldPath assigned to this array
     * @return the ldPath assigned to this array
     */
   public String getLDPath()
   {
      return ldPath;
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      if (getCachedActions() == null)
      {
         Vector actions = new Vector();

         actions.addElement(new HelpNowAction(this));
         actions.addElement(new NullAction());
         actions.addElement(new DeleteSpannedArrayAction(this));
         actions.addElement(new SyncArrayAction(this));
         actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_SPAN_ARRAY, getID()));
         actions.addElement(new PropertiesAction(this));

         setActions(actions);
      }

      return getCachedActions();
   }

   /** Returns true if some operation on this array has caused all actions to be blocked.
     * @return true if all actions should be blocked in the user interface
     */
   public boolean shouldBlockAllActions()
   {
      return hasProgress();
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public RaidObjectPropertySet getRaidObjectPropertySet()
   {
      RaidObjectPropertySet set = new RaidObjectPropertySet(getDisplayName(), "l_span.gif");
      RaidObjectPropertyGroup group = new RaidObjectPropertyGroup(getShortDisplayName());
      set.addGroup(group);
      set.setDisplayDataSetNames(getDisplayDataSetNames());

      Vector vLogicalDrives = getLogicalDriveCollection(null);

      group.addElement(new Object[] { new ToolTipString("infoArrayIdentifier"), getDisplayID()});
      group.addElement(new Object[] { new ToolTipString("infoSpannedArraySize"), String.valueOf(getSize()) + " " + JCRMUtil.getDisplayUnits()});

      if (vLogicalDrives.size() > 0)
         group.addElement(new Object[] { new ToolTipString("infoArrayFreeSpace"), String.valueOf(getFreeSpace()) + " " + JCRMUtil.getDisplayUnits()});

      group.addElement(new Object[] { new ToolTipString("infoArrayNumLogDrive"), String.valueOf(vLogicalDrives.size())});
      group.addElement(new Object[] { new ToolTipString("infoArrayNumPhysDrive"), String.valueOf(getPhysicalDeviceCollection(null).size())});

      return set;
   }           

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCSpannedArray";
   }

   /** Returns a shallow clone of this object, sans children.
     * @return a shallow clone of this object, sans children
     */
   public RaidObject emptyClone()
   {
      IrocSpannedArray array = new IrocSpannedArray(getAdapter(), 
                                                    getID(),
                                                    getSize(),
                                                    getFreeSpace(),
                                                    getUniqueID(), 
                                                    getStripeSize(), 
                                                    getLDPath());

      array.remove(getLogicalDrivesContainer());
      array.remove(getArraysContainer()); 
      return array;
   }
}



